﻿using System;
using System.IO;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.Collections.Generic;

namespace C_Sharp_Sample
{
    /// <summary>
    /// This class demonstrates the usage of .NET Win HTML Editor Control.
    /// </summary>
    public partial class Form1 : Form
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Form1"/> class.
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// This list holds the deserialized student data form the XML file. Finally this list will be
        /// bound to the BindingSource.
        /// </summary>
        private List<Student> myStudents = new List<Student>();


        /// <summary>
        /// The xml file where the student data is stored.
        /// </summary>
        readonly string studentXmlFile = string.Format("{0}\\students.xml", Application.StartupPath);


        /// <summary>
        /// This method serialize the student data in an XML file.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void studentBindingNavigatorSaveItem_Click(object sender, EventArgs e)
        {
            this.Validate();
            bindingSource2.EndEdit();

            XmlSerializer xs = new XmlSerializer(typeof(List<Student>));
            using (Stream s = File.Create(studentXmlFile))
                xs.Serialize(s, myStudents);
        }


        /// <summary>
        /// When the form is loaded, the student data is deserialized and myStudent field is populated.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Form1_Load(object sender, EventArgs e)
        {
            if (File.Exists(studentXmlFile))
            {
                try
                {
                    XmlSerializer xs = new XmlSerializer(typeof(List<Student>));
                    using (Stream s = File.OpenRead(studentXmlFile))
                        myStudents = (List<Student>)xs.Deserialize(s);
                }
                catch
                {
                }
            }

            bindingSource2.DataSource = myStudents;
        }
    }
}
