﻿
' ***********************************************************************
' Assembly         : WinFormHtmlEditor
' Author           : SpiceLogic
' Created          : 12-15-2012
'
' Last Modified By : SpiceLogic

' ***********************************************************************
' <copyright file="WinStyleBuilder.cs" company="SpiceLogic Inc.">
'     Copyright (c) SpiceLogic Inc.. All rights reserved.
' </copyright>
' <summary></summary>
' ***********************************************************************

Imports System.Collections.Generic
Imports System.Linq
Imports System.Reflection
Imports System.Text
Imports System.Windows.Forms
Imports SpiceLogic.HtmlEditorControl.Domain.BOs
Imports SpiceLogic.HtmlEditorControl.Domain.Dialogs

Namespace Dialogs.StyleBuilder
    ''' <summary>
    ''' Parses CSS style string and permits the user to edit it
    ''' </summary>
    Partial Public NotInheritable Class WinStyleBuilder
        Inherits Form
        Implements IStyleBuilderDialog

        ''' <summary>
        ''' User control currently displayed on the right
        ''' </summary>
        Private _selectedPage As Control

        ''' <summary>
        ''' Parsed style string
        ''' </summary>
        Private ReadOnly _propertiesDict As New Dictionary(Of String, String)()

#Region "-------- Constructors ----------"

        ''' <summary>
        ''' Initializes a new instance of the <see cref="WinStyleBuilder" /> class.
        ''' </summary>
        Public Sub New()
            InitializeComponent()
        End Sub

        '''// <summary>
        '''// Initializes a new instance of the <see cref="WinStyleBuilder"/> class.
        '''// </summary>
        '''// <param name="styleArg">The style arg.</param>
        '''// <param name="parsingTolerance">The parsing tolerance.</param>
        'public WinStyleBuilder(string styleArg, ParsingTolerence parsingTolerance)
        '{
        '    ParseStyleArgument(styleArg, parsingTolerance);
        '    InitializeComponent();
        '}

        '''// <summary>
        '''// Initializes a new instance of the <see cref="WinStyleBuilder"/> class.
        '''// </summary>
        '''// <param name="styleArg">The style arg.</param>
        'public WinStyleBuilder(string styleArg)
        '{
        '    ParseStyleArgument(styleArg, ParsingTolerence.NoTolerance);
        '    InitializeComponent();
        '}

#End Region

        ''' <summary>
        ''' Selects first page on load
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub frmFontSelector_Load(sender As Object, e As EventArgs) Handles Me.Load
            lbPage.SelectedIndex = 0
            lbPage_SelectedIndexChanged(sender, e)
        End Sub

        ''' <summary>
        ''' Updates the currenlty visible page
        ''' Iternally this method looks up current assembly using reflection for the appropriate page control to be shown
        ''' The page control should be marked with FormSelectorPageAttribute and have a constructor which accepts Dictionary&lt;string, string&gt;
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        ''' <exception cref="System.NotImplementedException"></exception>
        Private Sub lbPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles lbPage.SelectedIndexChanged
            ' Dispose currently selector control if any
            If _selectedPage IsNot Nothing Then
                pnlSelectedPage.Controls.Remove(_selectedPage)
                DirectCast(_selectedPage, IEditorStylePage).FlushContent()
                _selectedPage.Dispose()
                _selectedPage = Nothing
            End If

            ' Check selection and look for the control
            If lbPage.SelectedItem IsNot Nothing Then
                ' Name of the page
                Dim sName As String = DirectCast(lbPage.SelectedItem, String)

                ' Search this assembly for the appropriate control
                Dim aThisAssembly As Assembly = Assembly.GetAssembly(Me.[GetType]())
                For Each t As Type In aThisAssembly.GetTypes()
                    Dim attr As Attribute = Attribute.GetCustomAttribute(t, GetType(FormSelectorPageAttribute))
                    If attr IsNot Nothing Then
                        Dim pageAttr As FormSelectorPageAttribute = DirectCast(attr, FormSelectorPageAttribute)
                        If pageAttr.PageName = sName Then
                            _selectedPage = DirectCast(Activator.CreateInstance(t, _propertiesDict), Control)
                            Exit For
                        End If
                    End If
                Next

                ' Dummy check
                If _selectedPage Is Nothing Then
                    Throw New NotImplementedException(String.Format("Page {0} is not implemented in Style Page Selector.", sName))
                End If

                ' add it to the form
                pnlSelectedPage.SuspendLayout()
                pnlSelectedPage.Controls.Add(_selectedPage)
                _selectedPage.Dock = DockStyle.Fill
                pnlSelectedPage.ResumeLayout()
            End If
        End Sub

        ''' <summary>
        ''' Handles the FormClosed event of the frmFontSelector control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.Windows.Forms.FormClosedEventArgs" /> instance containing the event data.</param>
        Private Sub frmFontSelector_FormClosed(sender As Object, e As FormClosedEventArgs) Handles Me.FormClosed
            If _selectedPage IsNot Nothing Then
                pnlSelectedPage.Controls.Remove(_selectedPage)
                DirectCast(_selectedPage, IEditorStylePage).FlushContent()
                _selectedPage.Dispose()
                _selectedPage = Nothing
            End If
        End Sub

        ''' <summary>
        ''' Parses the style argument.
        ''' </summary>
        ''' <param name="styleArg">The style arg.</param>
        ''' <param name="parsingTolerance">The parsing tolerance.</param>
        ''' <exception cref="System.ArgumentException">
        ''' </exception>
        Public Sub ParseStyleArgument(styleArg As String, parsingTolerance As ParsingTolerence) Implements IStyleBuilderDialog.ParseStyleArgument
            Try
                Dim position As Integer = 0

                If String.IsNullOrEmpty(styleArg) Then
                    styleArg = String.Empty
                End If
                styleArg = styleArg.Trim()

                ' Handle enclosed quotes
                If styleArg.Length >= 2 AndAlso styleArg(0) = """"c AndAlso styleArg(styleArg.Length - 1) = """"c Then
                    styleArg = styleArg.Substring(1, styleArg.Length - 2).Trim()
                End If

                While position < styleArg.Length
                    Dim separatorI As Integer = styleArg.IndexOf(":"c, position)
                    If separatorI <> -1 Then
                        ' ':' found
                        ' localize tag
                        Dim tag As String = styleArg.Substring(position, separatorI - position).Trim().ToLowerInvariant()

                        ' localize parameters
                        Dim nextSemicolonI As Integer = styleArg.IndexOf(";"c, separatorI + 1)
                        If nextSemicolonI = -1 Then
                            nextSemicolonI = styleArg.Length
                        End If
                        Dim parameters As String = styleArg.Substring(separatorI + 1, nextSemicolonI - separatorI - 1).Trim()

                        ' store in dictionary
                        _propertiesDict(tag) = parameters

                        ' move starting position
                        position = nextSemicolonI + 1
                    Else
                        ' ':' not found
                        Dim reminder As String = styleArg.Substring(position).Trim()
                        If reminder.Length <> 0 AndAlso parsingTolerance <> ParsingTolerence.TolerateUnknownTags Then
                            Throw New ArgumentException(String.Format("Tag {0} has no specification", reminder))
                        End If

                        ' no exception was thrown - end parsing
                        Exit While
                    End If
                End While

                If parsingTolerance = ParsingTolerence.NoTolerance Then
                    ' create list of handling classes
                    Dim aThisAssembly As Assembly = Assembly.GetAssembly(Me.[GetType]())
                    Dim handlingClasses As List(Of FormSelectorPageAttribute) = aThisAssembly.GetTypes().[Select](Function(t) Attribute.GetCustomAttribute(t, GetType(FormSelectorPageAttribute))).Where(Function(attr) attr IsNot Nothing).Cast(Of FormSelectorPageAttribute)().ToList()

                    ' iterate through tags
                    For Each kvp As KeyValuePair(Of String, String) In _propertiesDict
                        Dim tagHandled As Boolean = handlingClasses.Any(Function(attr) attr.HandlesTag(kvp.Key))
                        ' search for a class to handle this attribute

                        If Not tagHandled Then
                            Throw New ArgumentException(String.Format("Tag {0} can't be handled.", kvp.Key))
                        End If
                    Next
                End If
            Catch err As Exception
                MessageBox.Show(err.Message, "Error parsing")
            End Try
        End Sub


#Region "------- Exposed Getters ----------------"

        ''' <summary>
        ''' Gets a value indicating whether [remove style].
        ''' </summary>
        ''' <value><c>true</c> if [remove style]; otherwise, <c>false</c>.</value>
        Public Property RemoveStyle() As Boolean Implements IStyleBuilderDialog.RemoveStyle
            Get
                Return chkRemoveStyle.Checked
            End Get
            Set(value As Boolean)
                chkRemoveStyle.Checked = value
            End Set
        End Property

        ''' <summary>
        ''' Gets the CSS text value.
        ''' </summary>
        ''' <value>The CSS text value.</value>
        Public ReadOnly Property CSSTextValue() As String Implements IStyleBuilderDialog.CSSTextValue
            Get
                ' iterate through properties and construct string representation
                Dim sb As New StringBuilder()
                For Each kvp As KeyValuePair(Of String, String) In _propertiesDict
                    If kvp.Value.Trim().Length > 0 Then
                        sb.Append(kvp.Key)
                        sb.Append(": ")
                        sb.Append(kvp.Value)
                        sb.Append("; ")
                    End If
                Next
                Return sb.ToString().Trim()
            End Get
        End Property

#End Region

        ''' <summary>
        ''' Shows current form as a dialog
        ''' </summary>
        ''' <returns>Returns <see cref="System.Windows.Forms.DialogResult"/></returns>
        Public Overloads Function ShowDialog() As DialogResult Implements IDialog.ShowDialog
            Return MyBase.ShowDialog()
        End Function

    End Class
End Namespace