﻿
' ***********************************************************************
' Assembly         : WinFormHtmlEditor
' Author           : SpiceLogic
' Created          : 12-15-2012
'
' Last Modified By : SpiceLogic

' ***********************************************************************
' <copyright file="ucBackground.cs" company="SpiceLogic Inc.">
'     Copyright (c) SpiceLogic Inc.. All rights reserved.
' </copyright>
' <summary></summary>
' ***********************************************************************

Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Drawing
Imports System.Text.RegularExpressions
Imports System.Windows.Forms

Namespace Dialogs.StyleBuilder
    ''' <summary>
    ''' Class ucBackground
    ''' </summary>
    <ToolboxItem(False)> _
    <FormSelectorPage("Background", "background-color,background-image,background-repeat,background-attachment,background-position-x,background-position-y")> _
    Partial Public Class ucBackground
        Inherits UserControl
        Implements IEditorStylePage

#Region "Preset of possible values"

        ''' <summary>
        ''' The _ bg repeat
        ''' </summary>
        ReadOnly _bgRepeat As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ bg attachment
        ''' </summary>
        ReadOnly _bgAttachment As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ bg position X
        ''' </summary>
        ReadOnly _bgPositionX As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ bg position Y
        ''' </summary>
        ReadOnly _bgPositionY As New List(Of KeyValuePair(Of String, String))()
#End Region

        ''' <summary>
        ''' The _dict
        ''' </summary>
        ReadOnly _dict As Dictionary(Of String, String)

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ucBackground"/> class.
        ''' </summary>
        ''' <param name="dict">The dict.</param>
        Public Sub New(dict As Dictionary(Of String, String))
            '#Region "Initialize presets"
            _bgRepeat.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _bgRepeat.Add(New KeyValuePair(Of String, String)("Tile in horizontal direction", "repeat-x"))
            _bgRepeat.Add(New KeyValuePair(Of String, String)("Tile in vertical direction", "repeat-y"))
            _bgRepeat.Add(New KeyValuePair(Of String, String)("Tile in both directions", "repeat"))
            _bgRepeat.Add(New KeyValuePair(Of String, String)("Do not tile", "no-repeat"))

            _bgAttachment.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _bgAttachment.Add(New KeyValuePair(Of String, String)("Scrolling background", "scroll"))
            _bgAttachment.Add(New KeyValuePair(Of String, String)("Fixed background", "fixed"))

            _bgPositionX.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _bgPositionX.Add(New KeyValuePair(Of String, String)("Left", "left"))
            _bgPositionX.Add(New KeyValuePair(Of String, String)("Center", "center"))
            _bgPositionX.Add(New KeyValuePair(Of String, String)("Right", "right"))
            _bgPositionX.Add(New KeyValuePair(Of String, String)("Custom", ""))

            _bgPositionY.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _bgPositionY.Add(New KeyValuePair(Of String, String)("Top", "top"))
            _bgPositionY.Add(New KeyValuePair(Of String, String)("Center", "center"))
            _bgPositionY.Add(New KeyValuePair(Of String, String)("Bottom", "bottom"))
            _bgPositionY.Add(New KeyValuePair(Of String, String)("Custom", ""))
            '#End Region

            _dict = dict
            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Handles the Load event of the ucBackground control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub ucBackground_Load(sender As Object, e As EventArgs) Handles Me.Load
            cbTiling.DataSource = _bgRepeat
            cbTiling.DisplayMember = "Key"
            cbTiling.ValueMember = "Value"

            cbScrolling.DataSource = _bgAttachment
            cbScrolling.DisplayMember = "Key"
            cbScrolling.ValueMember = "Value"

            cbHorizontal.DataSource = _bgPositionX
            cbHorizontal.DisplayMember = "Key"
            cbHorizontal.ValueMember = "Value"

            cbVertical.DataSource = _bgPositionY
            cbVertical.DisplayMember = "Key"
            cbVertical.ValueMember = "Value"

            Dim value As String

            If _dict.TryGetValue("background-color", value) Then
                If (If(value, "")).Trim().Equals("transparent", StringComparison.InvariantCultureIgnoreCase) Then
                    cbBgColorTransparent.Checked = True
                Else
                    txtBgColor.BackColor = ColorTranslator.FromHtml(value)
                    cbBgColorTransparent.Checked = False
                End If
            End If

            If _dict.TryGetValue("background-repeat", value) Then
                Dim val As String = value.Trim().ToLowerInvariant()
                For i As Integer = 0 To _bgRepeat.Count - 1
                    If String.Equals(val, _bgRepeat(i).Value) Then
                        cbTiling.SelectedIndex = i
                        Exit For
                    End If
                Next
            End If

            If _dict.TryGetValue("background-attachment", value) Then
                Dim val As String = value.Trim().ToLowerInvariant()
                For i As Integer = 0 To _bgAttachment.Count - 1
                    If String.Equals(val, _bgAttachment(i).Value) Then
                        cbScrolling.SelectedIndex = i
                        Exit For
                    End If
                Next
            End If

            If _dict.TryGetValue("background-position-x", value) Then
                Dim handled As Boolean = False

                Dim i As Integer = 0
                While i < _bgPositionX.Count AndAlso Not handled
                    If String.Equals(value, _bgPositionX(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbHorizontal.SelectedIndex = i
                        handled = True
                    End If
                    i += 1
                End While

                If Not handled Then
                    For i = 0 To cbHorCustType.Items.Count - 1
                        If value.EndsWith(DirectCast(cbHorCustType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                            cbHorizontal.SelectedIndex = 4
                            ' Custom
                            cbHorCustType.SelectedIndex = i
                            tbHorCust.Text = value.Substring(0, value.Length - DirectCast(cbHorCustType.Items(i), String).Length)
                            Exit For
                        End If
                    Next
                End If
            End If

            If _dict.TryGetValue("background-position-y", value) Then
                Dim handled As Boolean = False

                Dim i As Integer = 0
                While i < _bgPositionY.Count AndAlso Not handled
                    If String.Equals(value, _bgPositionY(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbVertical.SelectedIndex = i
                        handled = True
                    End If
                    i += 1
                End While

                If Not handled Then
                    For i = 0 To cbVerCustType.Items.Count - 1
                        If value.EndsWith(DirectCast(cbVerCustType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                            cbVertical.SelectedIndex = 4
                            ' Custom
                            cbVerCustType.SelectedIndex = i
                            tbVerCust.Text = value.Substring(0, value.Length - DirectCast(cbVerCustType.Items(i), String).Length)
                            Exit For
                        End If
                    Next
                End If
            End If

            If _dict.TryGetValue("background-image", value) Then
                If (If(value, "")).Trim().Equals("none", StringComparison.InvariantCultureIgnoreCase) Then
                    cbDoNotUseBackground.Checked = True
                Else
                    cbDoNotUseBackground.Checked = False
                    Try
                        tbBgImage.Text = Regex.Match(value, "url\((.+?)\)", RegexOptions.IgnoreCase).Groups(1).Value
                    Catch
                        tbBgImage.Text = value
                    End Try
                End If
            Else
                tbBgImage_TextChanged(Me, EventArgs.Empty)
            End If
        End Sub

#Region "UI handling"
        ''' <summary>
        ''' Handles the CheckedChanged event of the cbBgColorTransparent control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbBgColorTransparent_CheckedChanged(sender As Object, e As EventArgs) Handles cbBgColorTransparent.CheckedChanged
            txtBgColor.Enabled = Not cbBgColorTransparent.Checked
        End Sub

        ''' <summary>
        ''' Handles the TextChanged event of the tbBgImage control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub tbBgImage_TextChanged(sender As Object, e As EventArgs) Handles tbBgImage.TextChanged
            cbTiling.Enabled = InlineAssignHelper(cbScrolling.Enabled, InlineAssignHelper(gbPosition.Enabled, tbBgImage.Text.Trim().Length > 0))
        End Sub

        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbHorizontal control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbHorizontal_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbHorizontal.SelectedIndexChanged
            tbHorCust.Enabled = InlineAssignHelper(cbHorCustType.Enabled, cbHorizontal.SelectedIndex = 4)
        End Sub

        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbVertical control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbVertical_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbVertical.SelectedIndexChanged
            tbVerCust.Enabled = InlineAssignHelper(cbVerCustType.Enabled, cbVertical.SelectedIndex = 4)
        End Sub

        ''' <summary>
        ''' Handles the CheckedChanged event of the cbDoNotUseBackground control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbDoNotUseBackground_CheckedChanged(sender As Object, e As EventArgs) Handles cbDoNotUseBackground.CheckedChanged
            If cbDoNotUseBackground.Checked Then
                tbBgImage.Enabled = InlineAssignHelper(btChooseBgImage.Enabled, InlineAssignHelper(cbTiling.Enabled, InlineAssignHelper(cbScrolling.Enabled, InlineAssignHelper(gbPosition.Enabled, False))))
            Else
                tbBgImage.Enabled = InlineAssignHelper(btChooseBgImage.Enabled, InlineAssignHelper(cbTiling.Enabled, InlineAssignHelper(cbScrolling.Enabled, InlineAssignHelper(gbPosition.Enabled, True))))
                tbBgImage_TextChanged(Me, EventArgs.Empty)
            End If
        End Sub

        ''' <summary>
        ''' Handles the Click event of the btChooseBgImage control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub btChooseBgImage_Click(sender As Object, e As EventArgs) Handles btChooseBgImage.Click
            If dlgBgImage.ShowDialog() = DialogResult.OK Then
                tbBgImage.Text = dlgBgImage.FileName
            End If
        End Sub
#End Region

        ''' <summary>
        ''' Flushes the content of the user control back to the dictionary
        ''' </summary>
        Public Sub FlushContent() Implements IEditorStylePage.FlushContent
            _dict.Remove("background-color")
            _dict.Remove("background-image")
            _dict.Remove("background-repeat")
            _dict.Remove("background-attachment")
            _dict.Remove("background-position-x")
            _dict.Remove("background-position-y")

            _dict("background-color") = If(cbBgColorTransparent.Checked, "transparent", ColorTranslator.ToHtml(txtBgColor.BackColor))

            If cbDoNotUseBackground.Checked Then
                _dict("background-image") = "none"
            Else
                _dict("background-image") = If(String.IsNullOrEmpty(tbBgImage.Text.Trim()), String.Empty, String.Format("url({0})", tbBgImage.Text.Trim()))
                _dict("background-repeat") = DirectCast(cbTiling.SelectedValue, String)
                _dict("background-attachment") = DirectCast(cbScrolling.SelectedValue, String)

                If cbHorizontal.SelectedIndex <> 4 Then
                    _dict("background-position-x") = DirectCast(cbHorizontal.SelectedValue, String)
                Else
                    _dict("background-position-x") = tbHorCust.Text + cbHorCustType.Text
                End If

                If cbVertical.SelectedIndex <> 4 Then
                    _dict("background-position-y") = DirectCast(cbVertical.SelectedValue, String)
                Else
                    _dict("background-position-y") = tbVerCust.Text + cbVerCustType.Text
                End If
            End If
        End Sub

        ''' <summary>
        ''' Handles the Click event of the txtBgColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtBgColor_Click(sender As Object, e As EventArgs) Handles txtBgColor.Click
            If dlgColorPicker.ShowDialog() = DialogResult.OK Then
                txtBgColor.BackColor = dlgColorPicker.Color
            End If
        End Sub
        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function

    End Class
End Namespace