﻿
' ***********************************************************************
' Assembly         : WinFormHtmlEditor
' Author           : SpiceLogic
' Created          : 12-15-2012
'
' Last Modified By : SpiceLogic

' ***********************************************************************
' <copyright file="ucEdges.cs" company="SpiceLogic Inc.">
'     Copyright (c) SpiceLogic Inc.. All rights reserved.
' </copyright>
' <summary></summary>
' ***********************************************************************

Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Text
Imports System.Windows.Forms

Namespace Dialogs.StyleBuilder
    ''' <summary>
    ''' Class ucEdges
    ''' </summary>
    <ToolboxItem(False)> _
    <FormSelectorPage("Edges", "margin-top,margin-bottom,margin-left,margin-right,padding-top,padding-bottom,padding-left,padding-right,border-top-style,border-bottom-style,border-left-style,border-right-style")> _
    Partial Public Class ucEdges
        Inherits UserControl
        Implements IEditorStylePage

        ''' <summary>
        ''' The _dict
        ''' </summary>
        ReadOnly _dict As Dictionary(Of String, String)

#Region "Preset of possible value"

        ''' <summary>
        ''' The _ border top style
        ''' </summary>
        ReadOnly _borderTopStyle As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ border bottom style
        ''' </summary>
        ReadOnly _borderBottomStyle As List(Of KeyValuePair(Of String, String))
        ''' <summary>
        ''' The _ border left style
        ''' </summary>
        ReadOnly _borderLeftStyle As List(Of KeyValuePair(Of String, String))
        ''' <summary>
        ''' The _ border right style
        ''' </summary>
        ReadOnly _borderRightStyle As List(Of KeyValuePair(Of String, String))
#End Region

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ucEdges"/> class.
        ''' </summary>
        ''' <param name="dict">The dict.</param>
        Public Sub New(dict As Dictionary(Of String, String))
            _dict = dict

            '#Region "Initialize presets"
            _borderTopStyle.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _borderTopStyle.Add(New KeyValuePair(Of String, String)("None", "none"))
            _borderTopStyle.Add(New KeyValuePair(Of String, String)("Dotted", "dotted"))
            _borderTopStyle.Add(New KeyValuePair(Of String, String)("Dashed", "dashed"))
            _borderTopStyle.Add(New KeyValuePair(Of String, String)("Solid line", "solid"))
            _borderTopStyle.Add(New KeyValuePair(Of String, String)("Double line", "double"))
            _borderTopStyle.Add(New KeyValuePair(Of String, String)("Groove", "groove"))
            _borderTopStyle.Add(New KeyValuePair(Of String, String)("Ridge", "ridge"))
            _borderTopStyle.Add(New KeyValuePair(Of String, String)("Inset", "inset"))
            _borderTopStyle.Add(New KeyValuePair(Of String, String)("Outset", "outset"))

            _borderBottomStyle = New List(Of KeyValuePair(Of String, String))(_borderTopStyle)
            _borderLeftStyle = New List(Of KeyValuePair(Of String, String))(_borderTopStyle)
            _borderRightStyle = New List(Of KeyValuePair(Of String, String))(_borderTopStyle)
            '#End Region

            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Flushes the content of the user control back to the dictionary
        ''' </summary>
        Public Sub FlushContent() Implements IEditorStylePage.FlushContent
            _dict.Remove("margin-top")
            _dict.Remove("margin-bottom")
            _dict.Remove("margin-left")
            _dict.Remove("margin-right")
            _dict.Remove("padding-top")
            _dict.Remove("padding-bottom")
            _dict.Remove("padding-left")
            _dict.Remove("padding-right")

            _dict.Remove("border-top-style")
            _dict.Remove("border-bottom-style")
            _dict.Remove("border-left-style")
            _dict.Remove("border-right-style")

            If tbMTop.Text.Trim().Length > 0 Then
                _dict("margin-top") = String.Concat(tbMTop.Text, cbMTopType.Text)
            End If
            If tbMBottom.Text.Trim().Length > 0 Then
                _dict("margin-bottom") = String.Concat(tbMBottom.Text, cbMBottomType.Text)
            End If
            If tbMLeft.Text.Trim().Length > 0 Then
                _dict("margin-left") = String.Concat(tbMLeft.Text, cbMLeftType.Text)
            End If
            If tbMRight.Text.Trim().Length > 0 Then
                _dict("margin-right") = String.Concat(tbMRight.Text, cbMRightType.Text)
            End If

            If tbPTop.Text.Trim().Length > 0 Then
                _dict("padding-top") = String.Concat(tbPTop.Text, cbPTopType.Text)
            End If
            If tbPBottom.Text.Trim().Length > 0 Then
                _dict("padding-bottom") = String.Concat(tbPBottom.Text, cbPBottomType.Text)
            End If
            If tbPLeft.Text.Trim().Length > 0 Then
                _dict("padding-left") = String.Concat(tbPLeft.Text, cbPLeftType.Text)
            End If
            If tbPRight.Text.Trim().Length > 0 Then
                _dict("padding-right") = String.Concat(tbPRight.Text, cbPRightType.Text)
            End If

            If True Then
                ' left border
                Dim sb As New StringBuilder()
                If cbLeftStyle.SelectedIndex > 0 Then
                    sb.Append(DirectCast(cbLeftStyle.SelectedValue, String))
                    sb.Append(" "c)

                    If cbLeftWidth.SelectedIndex > 0 Then
                        If cbLeftWidth.SelectedIndex = 4 Then
                            If tbLeftWidth.Text.Trim().Length > 0 Then
                                sb.Append(String.Concat(tbLeftWidth.Text.Trim(), cbLeftWidthType.Text))
                                sb.Append(" "c)
                            End If
                        Else
                            sb.Append(cbLeftWidth.Text.ToLowerInvariant())
                            sb.Append(" "c)
                        End If
                    End If

                    sb.Append(cbLeftColor.Text.ToLowerInvariant())
                End If
                _dict("border-left-style") = sb.ToString()
            End If

            If True Then
                ' right border
                Dim sb As New StringBuilder()
                If cbRightStyle.SelectedIndex > 0 Then
                    sb.Append(DirectCast(cbRightStyle.SelectedValue, String))
                    sb.Append(" "c)

                    If cbRightWidth.SelectedIndex > 0 Then
                        If cbRightWidth.SelectedIndex = 4 Then
                            If tbRightWidth.Text.Trim().Length > 0 Then
                                sb.Append(String.Concat(tbRightWidth.Text.Trim(), cbRightWidthType.Text))
                                sb.Append(" "c)
                            End If
                        Else
                            sb.Append(cbRightWidth.Text.ToLowerInvariant())
                            sb.Append(" "c)
                        End If
                    End If

                    sb.Append(cbRightColor.Text.ToLowerInvariant())
                End If
                _dict("border-right-style") = sb.ToString()
            End If

            If True Then
                ' top border
                Dim sb As New StringBuilder()
                If cbTopStyle.SelectedIndex > 0 Then
                    sb.Append(DirectCast(cbTopStyle.SelectedValue, String))
                    sb.Append(" "c)

                    If cbTopWidth.SelectedIndex > 0 Then
                        If cbTopWidth.SelectedIndex = 4 Then
                            If tbTopWidth.Text.Trim().Length > 0 Then
                                sb.Append(String.Concat(tbTopWidth.Text.Trim(), cbTopWidthType.Text))
                                sb.Append(" "c)
                            End If
                        Else
                            sb.Append(cbTopWidth.Text.ToLowerInvariant())
                            sb.Append(" "c)
                        End If
                    End If

                    sb.Append(cbTopColor.Text.ToLowerInvariant())
                End If
                _dict("border-top-style") = sb.ToString()
            End If

            If True Then
                ' bottom border
                Dim sb As New StringBuilder()
                If cbBottomStyle.SelectedIndex > 0 Then
                    sb.Append(DirectCast(cbBottomStyle.SelectedValue, String))
                    sb.Append(" "c)

                    If cbBottomWidth.SelectedIndex > 0 Then
                        If cbBottomWidth.SelectedIndex = 4 Then
                            If tbBottomWidth.Text.Trim().Length > 0 Then
                                sb.Append(String.Concat(tbBottomWidth.Text.Trim(), cbBottomWidthType.Text))
                                sb.Append(" "c)
                            End If
                        Else
                            sb.Append(cbBottomWidth.Text.ToLowerInvariant())
                            sb.Append(" "c)
                        End If
                    End If

                    sb.Append(cbBottomColor.Text.ToLowerInvariant())
                End If
                _dict("border-bottom-style") = sb.ToString()
            End If
        End Sub

        ''' <summary>
        ''' Handles the Load event of the ucEdges control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub ucEdges_Load(sender As Object, e As EventArgs) Handles Me.Load
            '#Region "set data sources"
            cbLeftStyle.DataSource = _borderLeftStyle
            cbLeftStyle.DisplayMember = "Key"
            cbLeftStyle.ValueMember = "Value"

            cbRightStyle.DataSource = _borderRightStyle
            cbRightStyle.DisplayMember = "Key"
            cbRightStyle.ValueMember = "Value"

            cbTopStyle.DataSource = _borderTopStyle
            cbTopStyle.DisplayMember = "Key"
            cbTopStyle.ValueMember = "Value"

            cbBottomStyle.DataSource = _borderBottomStyle
            cbBottomStyle.DisplayMember = "Key"
            cbBottomStyle.ValueMember = "Value"

            cbLeftWidth.SelectedIndex = 0
            cbRightWidth.SelectedIndex = 0
            cbTopWidth.SelectedIndex = 0
            cbBottomWidth.SelectedIndex = 0
            '#End Region

            Dim value As String

            '#Region "parse margins"
            If _dict.TryGetValue("margin-top", value) Then
                Dim i As Integer = 0, n As Integer = cbMTopType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbMTopType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        cbMTopType.SelectedIndex = i
                        tbMTop.Text = value.Substring(0, value.Length - DirectCast(cbMTopType.Items(i), String).Length)
                        Exit While
                    End If
                    i += 1
                End While
            End If
            If _dict.TryGetValue("margin-bottom", value) Then
                Dim i As Integer = 0, n As Integer = cbMBottomType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbMBottomType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        cbMBottomType.SelectedIndex = i
                        tbMBottom.Text = value.Substring(0, value.Length - DirectCast(cbMBottomType.Items(i), String).Length)
                        Exit While
                    End If
                    i += 1
                End While
            End If
            If _dict.TryGetValue("margin-left", value) Then
                Dim i As Integer = 0, n As Integer = cbMLeftType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbMLeftType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        cbMLeftType.SelectedIndex = i
                        tbMLeft.Text = value.Substring(0, value.Length - DirectCast(cbMLeftType.Items(i), String).Length)
                        Exit While
                    End If
                    i += 1
                End While
            End If
            If _dict.TryGetValue("margin-right", value) Then
                Dim i As Integer = 0, n As Integer = cbMRightType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbMRightType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        cbMRightType.SelectedIndex = i
                        tbMRight.Text = value.Substring(0, value.Length - DirectCast(cbMRightType.Items(i), String).Length)
                        Exit While
                    End If
                    i += 1
                End While
            End If
            '#End Region

            '#Region "parse padding"
            If _dict.TryGetValue("padding-top", value) Then
                Dim i As Integer = 0, n As Integer = cbPTopType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbPTopType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        cbPTopType.SelectedIndex = i
                        tbPTop.Text = value.Substring(0, value.Length - DirectCast(cbPTopType.Items(i), String).Length)
                        Exit While
                    End If
                    i += 1
                End While
            End If
            If _dict.TryGetValue("padding-bottom", value) Then
                Dim i As Integer = 0, n As Integer = cbPBottomType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbPBottomType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        cbPBottomType.SelectedIndex = i
                        tbPBottom.Text = value.Substring(0, value.Length - DirectCast(cbPBottomType.Items(i), String).Length)
                        Exit While
                    End If
                    i += 1
                End While
            End If
            If _dict.TryGetValue("padding-left", value) Then
                Dim i As Integer = 0, n As Integer = cbPLeftType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbPLeftType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        cbPLeftType.SelectedIndex = i
                        tbPLeft.Text = value.Substring(0, value.Length - DirectCast(cbPLeftType.Items(i), String).Length)
                        Exit While
                    End If
                    i += 1
                End While
            End If
            If _dict.TryGetValue("padding-right", value) Then
                Dim i As Integer = 0, n As Integer = cbPRightType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbPRightType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        cbPRightType.SelectedIndex = i
                        tbPRight.Text = value.Substring(0, value.Length - DirectCast(cbPRightType.Items(i), String).Length)
                        Exit While
                    End If
                    i += 1
                End While
            End If
            '#End Region

            '#Region "parse left border"
            If _dict.TryGetValue("border-left-style", value) Then
                Dim values As New List(Of String)(value.Split(" "c))
                ' Filter empty

                Dim i As Integer = 0
                While i < values.Count
                    If values(i).Trim().Length = 0 Then
                        values.RemoveAt(i)
                        i -= 1
                    End If

                    i += 1
                End While

                Dim styleFound As Boolean = False

                Dim valI As Integer = 0
                While valI < values.Count AndAlso Not styleFound
                    value = values(valI)
                    i = 0
                    Dim n As Integer = _borderLeftStyle.Count
                    While i < n AndAlso Not styleFound
                        If String.Equals(value, _borderLeftStyle(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                            cbLeftStyle.SelectedIndex = i
                            values.RemoveAt(valI)
                            styleFound = True
                        End If
                        i += 1
                    End While
                    valI += 1
                End While

                If styleFound AndAlso cbLeftStyle.SelectedIndex >= 2 Then
                    Dim widthFound As Boolean = False

                    valI = 0
                    While valI < values.Count AndAlso Not widthFound
                        value = values(valI)
                        i = 1
                        While i < 4 AndAlso Not widthFound
                            If String.Equals(value, DirectCast(cbLeftWidth.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                cbLeftWidth.SelectedIndex = i
                                values.RemoveAt(valI)
                                widthFound = True
                            End If
                            i += 1
                        End While
                        valI += 1
                    End While

                    If Not widthFound Then
                        valI = 0
                        While valI < values.Count AndAlso Not widthFound
                            value = values(valI)
                            i = 0
                            Dim n As Integer = cbLeftWidthType.Items.Count
                            While i < n AndAlso Not widthFound
                                If value.EndsWith(DirectCast(cbLeftWidthType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                    cbLeftWidth.SelectedIndex = 4
                                    cbLeftWidthType.SelectedIndex = i
                                    tbLeftWidth.Text = value.Substring(0, value.Length - DirectCast(cbLeftWidthType.Items(i), String).Length)
                                    values.RemoveAt(valI)
                                    widthFound = True
                                End If
                                i += 1
                            End While
                            valI += 1
                        End While
                    End If

                    Dim colorFound As Boolean = False

                    valI = 0
                    While valI < values.Count AndAlso Not colorFound
                        value = values(valI)
                        i = 0
                        Dim n As Integer = cbLeftColor.Items.Count
                        While i < n AndAlso Not colorFound
                            If String.Equals(value, DirectCast(cbLeftColor.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                cbLeftColor.SelectedIndex = i
                                values.RemoveAt(valI)
                                colorFound = True
                            End If
                            i += 1
                        End While
                        valI += 1
                    End While

                    If Not colorFound Then
                        valI = 0
                        While valI < values.Count AndAlso Not colorFound
                            value = values(valI)
                            If value.StartsWith("#") Then
                                cbLeftColor.Text = value
                                values.RemoveAt(valI)
                                colorFound = True
                            End If
                            valI += 1
                        End While
                    End If
                End If
            End If
            '#End Region

            '#Region "parse Right border"
            If _dict.TryGetValue("border-right-style", value) Then
                Dim values As New List(Of String)(value.Split(" "c))
                ' Filter empty

                Dim i As Integer = 0
                While i < values.Count
                    If values(i).Trim().Length = 0 Then
                        values.RemoveAt(i)
                        i -= 1
                    End If

                    i += 1
                End While

                Dim styleFound As Boolean = False

                Dim valI As Integer = 0
                While valI < values.Count AndAlso Not styleFound
                    value = values(valI)
                    i = 0
                    Dim n As Integer = _borderRightStyle.Count
                    While i < n AndAlso Not styleFound
                        If String.Equals(value, _borderRightStyle(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                            cbRightStyle.SelectedIndex = i
                            values.RemoveAt(valI)
                            styleFound = True
                        End If
                        i += 1
                    End While
                    valI += 1
                End While

                If styleFound AndAlso cbRightStyle.SelectedIndex >= 2 Then
                    Dim widthFound As Boolean = False

                    valI = 0
                    While valI < values.Count AndAlso Not widthFound
                        value = values(valI)
                        i = 1
                        While i < 4 AndAlso Not widthFound
                            If String.Equals(value, DirectCast(cbRightWidth.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                cbRightWidth.SelectedIndex = i
                                values.RemoveAt(valI)
                                widthFound = True
                            End If
                            i += 1
                        End While
                        valI += 1
                    End While

                    If Not widthFound Then
                        valI = 0
                        While valI < values.Count AndAlso Not widthFound
                            value = values(valI)
                            i = 0
                            Dim n As Integer = cbRightWidthType.Items.Count
                            While i < n AndAlso Not widthFound
                                If value.EndsWith(DirectCast(cbRightWidthType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                    cbRightWidth.SelectedIndex = 4
                                    cbRightWidthType.SelectedIndex = i
                                    tbRightWidth.Text = value.Substring(0, value.Length - DirectCast(cbRightWidthType.Items(i), String).Length)
                                    values.RemoveAt(valI)
                                    widthFound = True
                                End If
                                i += 1
                            End While
                            valI += 1
                        End While
                    End If

                    Dim colorFound As Boolean = False

                    valI = 0
                    While valI < values.Count AndAlso Not colorFound
                        value = values(valI)
                        i = 0
                        Dim n As Integer = cbRightColor.Items.Count
                        While i < n AndAlso Not colorFound
                            If String.Equals(value, DirectCast(cbRightColor.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                cbRightColor.SelectedIndex = i
                                values.RemoveAt(valI)
                                colorFound = True
                            End If
                            i += 1
                        End While
                        valI += 1
                    End While

                    If Not colorFound Then
                        valI = 0
                        While valI < values.Count AndAlso Not colorFound
                            value = values(valI)
                            If value.StartsWith("#") Then
                                cbRightColor.Text = value
                                values.RemoveAt(valI)
                                colorFound = True
                            End If
                            valI += 1
                        End While
                    End If
                End If
            End If
            '#End Region

            '#Region "parse Top border"
            If _dict.TryGetValue("border-top-style", value) Then
                Dim values As New List(Of String)(value.Split(" "c))
                ' Filter empty

                Dim i As Integer = 0
                While i < values.Count
                    If values(i).Trim().Length = 0 Then
                        values.RemoveAt(i)
                        i -= 1
                    End If

                    i += 1
                End While

                Dim styleFound As Boolean = False

                Dim valI As Integer = 0
                While valI < values.Count AndAlso Not styleFound
                    value = values(valI)
                    i = 0
                    Dim n As Integer = _borderTopStyle.Count
                    While i < n AndAlso Not styleFound
                        If String.Equals(value, _borderTopStyle(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                            cbTopStyle.SelectedIndex = i
                            values.RemoveAt(valI)
                            styleFound = True
                        End If
                        i += 1
                    End While
                    valI += 1
                End While

                If styleFound AndAlso cbTopStyle.SelectedIndex >= 2 Then
                    Dim widthFound As Boolean = False

                    valI = 0
                    While valI < values.Count AndAlso Not widthFound
                        value = values(valI)
                        i = 1
                        While i < 4 AndAlso Not widthFound
                            If String.Equals(value, DirectCast(cbTopWidth.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                cbTopWidth.SelectedIndex = i
                                values.RemoveAt(valI)
                                widthFound = True
                            End If
                            i += 1
                        End While
                        valI += 1
                    End While

                    If Not widthFound Then
                        valI = 0
                        While valI < values.Count AndAlso Not widthFound
                            value = values(valI)
                            i = 0
                            Dim n As Integer = cbTopWidthType.Items.Count
                            While i < n AndAlso Not widthFound
                                If value.EndsWith(DirectCast(cbTopWidthType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                    cbTopWidth.SelectedIndex = 4
                                    cbTopWidthType.SelectedIndex = i
                                    tbTopWidth.Text = value.Substring(0, value.Length - DirectCast(cbTopWidthType.Items(i), String).Length)
                                    values.RemoveAt(valI)
                                    widthFound = True
                                End If
                                i += 1
                            End While
                            valI += 1
                        End While
                    End If

                    Dim colorFound As Boolean = False

                    valI = 0
                    While valI < values.Count AndAlso Not colorFound
                        value = values(valI)
                        i = 0
                        Dim n As Integer = cbTopColor.Items.Count
                        While i < n AndAlso Not colorFound
                            If String.Equals(value, DirectCast(cbTopColor.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                cbTopColor.SelectedIndex = i
                                values.RemoveAt(valI)
                                colorFound = True
                            End If
                            i += 1
                        End While
                        valI += 1
                    End While

                    If Not colorFound Then
                        valI = 0
                        While valI < values.Count AndAlso Not colorFound
                            value = values(valI)
                            If value.StartsWith("#") Then
                                cbTopColor.Text = value
                                values.RemoveAt(valI)
                                colorFound = True
                            End If
                            valI += 1
                        End While
                    End If
                End If
            End If
            '#End Region

            '#Region "parse Bottom border"
            If _dict.TryGetValue("border-bottom-style", value) Then
                Dim values As New List(Of String)(value.Split(" "c))
                ' Filter empty

                Dim i As Integer = 0
                While i < values.Count
                    If values(i).Trim().Length = 0 Then
                        values.RemoveAt(i)
                        i -= 1
                    End If

                    i += 1
                End While

                Dim styleFound As Boolean = False

                Dim valI As Integer = 0
                While valI < values.Count AndAlso Not styleFound
                    value = values(valI)
                    i = 0
                    Dim n As Integer = _borderBottomStyle.Count
                    While i < n AndAlso Not styleFound
                        If String.Equals(value, _borderBottomStyle(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                            cbBottomStyle.SelectedIndex = i
                            values.RemoveAt(valI)
                            styleFound = True
                        End If
                        i += 1
                    End While
                    valI += 1
                End While

                If styleFound AndAlso cbBottomStyle.SelectedIndex >= 2 Then
                    Dim widthFound As Boolean = False

                    valI = 0
                    While valI < values.Count AndAlso Not widthFound
                        value = values(valI)
                        i = 1
                        While i < 4 AndAlso Not widthFound
                            If String.Equals(value, DirectCast(cbBottomWidth.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                cbBottomWidth.SelectedIndex = i
                                values.RemoveAt(valI)
                                widthFound = True
                            End If
                            i += 1
                        End While
                        valI += 1
                    End While

                    If Not widthFound Then
                        valI = 0
                        While valI < values.Count AndAlso Not widthFound
                            value = values(valI)
                            i = 0
                            Dim n As Integer = cbBottomWidthType.Items.Count
                            While i < n AndAlso Not widthFound
                                If value.EndsWith(DirectCast(cbBottomWidthType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                    cbBottomWidth.SelectedIndex = 4
                                    cbBottomWidthType.SelectedIndex = i
                                    tbBottomWidth.Text = value.Substring(0, value.Length - DirectCast(cbBottomWidthType.Items(i), String).Length)
                                    values.RemoveAt(valI)
                                    widthFound = True
                                End If
                                i += 1
                            End While
                            valI += 1
                        End While
                    End If

                    Dim colorFound As Boolean = False

                    valI = 0
                    While valI < values.Count AndAlso Not colorFound
                        value = values(valI)
                        i = 0
                        Dim n As Integer = cbBottomColor.Items.Count
                        While i < n AndAlso Not colorFound
                            If String.Equals(value, DirectCast(cbBottomColor.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                cbBottomColor.SelectedIndex = i
                                values.RemoveAt(valI)
                                colorFound = True
                            End If
                            i += 1
                        End While
                        valI += 1
                    End While

                    If Not colorFound Then
                        valI = 0
                        While valI < values.Count AndAlso Not colorFound
                            value = values(valI)
                            If value.StartsWith("#") Then
                                cbBottomColor.Text = value
                                values.RemoveAt(valI)
                                colorFound = True
                            End If
                            valI += 1
                        End While
                    End If
                End If
            End If
            '#End Region
        End Sub

#Region "left border edge handlers"
        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbLeftWidth control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbLeftWidth_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbLeftWidth.SelectedIndexChanged
            tbLeftWidth.Enabled = InlineAssignHelper(cbLeftWidthType.Enabled, cbLeftWidth.SelectedIndex = 4)
        End Sub

        ''' <summary>
        ''' Handles the Click event of the btLeftColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub btLeftColor_Click(sender As Object, e As EventArgs) Handles btLeftColor.Click
            If dlgColor.ShowDialog() = DialogResult.OK Then
                cbLeftColor.Text = String.Format("#{0}", (dlgColor.Color.ToArgb() And &HFFFFF).ToString("X6"))
            End If
        End Sub

        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbLeftStyle control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbLeftStyle_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbLeftStyle.SelectedIndexChanged
            If cbLeftStyle.SelectedIndex >= 2 Then
                cbLeftWidth.Enabled = InlineAssignHelper(tbLeftWidth.Enabled, InlineAssignHelper(cbLeftWidthType.Enabled, InlineAssignHelper(cbLeftColor.Enabled, InlineAssignHelper(btLeftColor.Enabled, True))))
                cbLeftWidth_SelectedIndexChanged(Me, EventArgs.Empty)
            Else
                cbLeftWidth.Enabled = InlineAssignHelper(tbLeftWidth.Enabled, InlineAssignHelper(cbLeftWidthType.Enabled, InlineAssignHelper(cbLeftColor.Enabled, InlineAssignHelper(btLeftColor.Enabled, False))))
            End If
        End Sub
#End Region

#Region "Right border edge handlers"
        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbRightWidth control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbRightWidth_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbRightWidth.SelectedIndexChanged
            tbRightWidth.Enabled = InlineAssignHelper(cbRightWidthType.Enabled, cbRightWidth.SelectedIndex = 4)
        End Sub

        ''' <summary>
        ''' Handles the Click event of the btRightColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub btRightColor_Click(sender As Object, e As EventArgs) Handles btRightColor.Click
            If dlgColor.ShowDialog() = DialogResult.OK Then
                cbRightColor.Text = String.Format("#{0}", (dlgColor.Color.ToArgb() And &HFFFFF).ToString("X6"))
            End If
        End Sub

        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbRightStyle control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbRightStyle_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbRightStyle.SelectedIndexChanged
            If cbRightStyle.SelectedIndex >= 2 Then
                cbRightWidth.Enabled = InlineAssignHelper(tbRightWidth.Enabled, InlineAssignHelper(cbRightWidthType.Enabled, InlineAssignHelper(cbRightColor.Enabled, InlineAssignHelper(btRightColor.Enabled, True))))
                cbRightWidth_SelectedIndexChanged(Me, EventArgs.Empty)
            Else
                cbRightWidth.Enabled = InlineAssignHelper(tbRightWidth.Enabled, InlineAssignHelper(cbRightWidthType.Enabled, InlineAssignHelper(cbRightColor.Enabled, InlineAssignHelper(btRightColor.Enabled, False))))
            End If
        End Sub
#End Region

#Region "Top border edge handlers"
        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbTopWidth control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbTopWidth_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbTopWidth.SelectedIndexChanged
            tbTopWidth.Enabled = InlineAssignHelper(cbTopWidthType.Enabled, cbTopWidth.SelectedIndex = 4)
        End Sub

        ''' <summary>
        ''' Handles the Click event of the btTopColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub btTopColor_Click(sender As Object, e As EventArgs) Handles btTopColor.Click
            If dlgColor.ShowDialog() = DialogResult.OK Then
                cbTopColor.Text = String.Format("#{0}", (dlgColor.Color.ToArgb() And &HFFFFF).ToString("X6"))
            End If
        End Sub

        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbTopStyle control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbTopStyle_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbTopStyle.SelectedIndexChanged
            If cbTopStyle.SelectedIndex >= 2 Then
                cbTopWidth.Enabled = InlineAssignHelper(tbTopWidth.Enabled, InlineAssignHelper(cbTopWidthType.Enabled, InlineAssignHelper(cbTopColor.Enabled, InlineAssignHelper(btTopColor.Enabled, True))))
                cbTopWidth_SelectedIndexChanged(Me, EventArgs.Empty)
            Else
                cbTopWidth.Enabled = InlineAssignHelper(tbTopWidth.Enabled, InlineAssignHelper(cbTopWidthType.Enabled, InlineAssignHelper(cbTopColor.Enabled, InlineAssignHelper(btTopColor.Enabled, False))))
            End If
        End Sub
#End Region

#Region "Bottom border edge handlers"
        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbBottomWidth control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbBottomWidth_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbBottomWidth.SelectedIndexChanged
            tbBottomWidth.Enabled = InlineAssignHelper(cbBottomWidthType.Enabled, cbBottomWidth.SelectedIndex = 4)
        End Sub

        ''' <summary>
        ''' Handles the Click event of the btBottomColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub btBottomColor_Click(sender As Object, e As EventArgs) Handles btBottomColor.Click
            If dlgColor.ShowDialog() = DialogResult.OK Then
                cbBottomColor.Text = String.Format("#{0}", (dlgColor.Color.ToArgb() And &HFFFFF).ToString("X6"))
            End If
        End Sub

        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbBottomStyle control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbBottomStyle_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbBottomStyle.SelectedIndexChanged
            If cbBottomStyle.SelectedIndex >= 2 Then
                cbBottomWidth.Enabled = InlineAssignHelper(tbBottomWidth.Enabled, InlineAssignHelper(cbBottomWidthType.Enabled, InlineAssignHelper(cbBottomColor.Enabled, InlineAssignHelper(btBottomColor.Enabled, True))))
                cbBottomWidth_SelectedIndexChanged(Me, EventArgs.Empty)
            Else
                cbBottomWidth.Enabled = InlineAssignHelper(tbBottomWidth.Enabled, InlineAssignHelper(cbBottomWidthType.Enabled, InlineAssignHelper(cbBottomColor.Enabled, InlineAssignHelper(btBottomColor.Enabled, False))))
            End If
        End Sub

        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function
#End Region

    End Class
End Namespace