﻿
' ***********************************************************************
' Assembly         : WinFormHtmlEditor
' Author           : SpiceLogic
' Created          : 12-15-2012
'
' Last Modified By : SpiceLogic

' ***********************************************************************
' <copyright file="ucFont.cs" company="SpiceLogic Inc.">
'     Copyright (c) SpiceLogic Inc.. All rights reserved.
' </copyright>
' <summary></summary>
' ***********************************************************************

Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms

Namespace Dialogs.StyleBuilder
    ''' <summary>
    ''' Class ucFont
    ''' </summary>
    <ToolboxItem(False)> _
    <FormSelectorPage("Font", "font;font-family;font-size;text-decoration;font-weight;text-transform;color;font-style;font-variant")> _
    Partial Public Class ucFont
        Inherits UserControl
        Implements IEditorStylePage

        ''' <summary>
        ''' The _dict
        ''' </summary>
        ReadOnly _dict As Dictionary(Of String, String)

#Region "Preset of possible value"

        ''' <summary>
        ''' The _ system fonts
        ''' </summary>
        ReadOnly _SystemFonts As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ font style
        ''' </summary>
        ReadOnly _FontStyle As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ font variant
        ''' </summary>
        ReadOnly _FontVariant As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ bold absolute
        ''' </summary>
        ReadOnly _BoldAbsolute As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ bold relative
        ''' </summary>
        ReadOnly _BoldRelative As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ text transform
        ''' </summary>
        ReadOnly _TextTransform As New List(Of KeyValuePair(Of String, String))()
#End Region

        ''' <summary>
        ''' Creates the lists.
        ''' </summary>
        Protected Sub CreateLists()
            '#Region "Initialize presets"
            _SystemFonts.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("Window caption", "caption"))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("ToolWindow caption", "small-caption"))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("Dialog text", "message-box"))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("Icon labels", "icon"))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("Menu text", "menu"))
            _SystemFonts.Add(New KeyValuePair(Of String, String)("Tooltip text", "status-bar"))

            _FontStyle.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _FontStyle.Add(New KeyValuePair(Of String, String)("Normal", "normal"))
            _FontStyle.Add(New KeyValuePair(Of String, String)("Italic", "italic"))

            _FontVariant.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _FontVariant.Add(New KeyValuePair(Of String, String)("Normal", "normal"))
            _FontVariant.Add(New KeyValuePair(Of String, String)("Small Caps", "small-caps"))

            _BoldAbsolute.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _BoldAbsolute.Add(New KeyValuePair(Of String, String)("Normal", "normal"))
            _BoldAbsolute.Add(New KeyValuePair(Of String, String)("Bold", "bold"))

            _BoldRelative.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _BoldRelative.Add(New KeyValuePair(Of String, String)("Lighter", "lighter"))
            _BoldRelative.Add(New KeyValuePair(Of String, String)("Bolder", "bolder"))

            _TextTransform.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _TextTransform.Add(New KeyValuePair(Of String, String)("None", "none"))
            _TextTransform.Add(New KeyValuePair(Of String, String)("Initial Cap", "capitalize"))
            _TextTransform.Add(New KeyValuePair(Of String, String)("lowercase", "lowercase"))
            _TextTransform.Add(New KeyValuePair(Of String, String)("UPPERCASE", "uppercase"))
            '#End Region
        End Sub

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ucFont"/> class.
        ''' </summary>
        ''' <param name="dict">The dict.</param>
        Public Sub New(dict As Dictionary(Of String, String))
            _dict = dict
            CreateLists()
            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Flushes the content of the user control back to the dictionary
        ''' </summary>
        Public Sub FlushContent() Implements IEditorStylePage.FlushContent
            _dict.Remove("font-family")
            _dict.Remove("font")
            _dict.Remove("color")
            _dict.Remove("font-style")
            _dict.Remove("font-variant")
            _dict.Remove("font-weight")
            _dict.Remove("font-size")
            _dict.Remove("text-decoration")

            If True Then
                ' handle text decoration
                Dim sb As New StringBuilder()
                If cbEffectNone.Checked Then
                    sb.Append(" none")
                End If
                If cbEffectUnderline.Checked Then
                    sb.Append(" underline")
                End If
                If cbEffectStrikethrough.Checked Then
                    sb.Append(" line-through")
                End If
                If cbEffectOverline.Checked Then
                    sb.Append(" overline")
                End If
                _dict("text-decoration") = sb.ToString()
            End If

            _dict("text-transform") = DirectCast(cbCapitalization.SelectedValue, String)

            If rbFamily.Checked Then
                _dict("font-family") = tbFontFamily.Text

                _dict("color") = ColorTranslator.ToHtml(txtForeColor.BackColor)
                ' cbColor.Text.ToLowerInvariant();
                _dict("font-style") = DirectCast(cbFontStyle.SelectedValue, String)
                _dict("font-variant") = DirectCast(cbFontVariant.SelectedValue, String)

                ' font-size
                If rbSizeSpecific.Checked Then
                    If tbSpecificSize.Text.Trim().Length > 0 Then
                        _dict("font-size") = tbSpecificSize.Text + cbSpecificSizeType.Text
                    End If
                ElseIf rbSizeAbsolute.Checked Then
                    _dict("font-size") = cbAbsoluteSize.Text.ToLowerInvariant()
                Else
                    _dict("font-size") = cbRelativeSize.Text.ToLowerInvariant()
                End If

                ' font-weight
                If rbBoldAbsolute.Checked Then
                    _dict("font-weight") = DirectCast(cbBoldAbsolute.SelectedValue, String)
                Else
                    _dict("font-weight") = DirectCast(cbBoldRelative.SelectedValue, String)
                End If
            End If

            If rbSystemFont.Checked Then
                _dict("font") = DirectCast(cbSystemFont.SelectedValue, String)
            End If
        End Sub

        ''' <summary>
        ''' Handles the Load event of the ucFont control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub ucFont_Load(sender As Object, e As EventArgs) Handles Me.Load
            '#Region "set data sources"
            cbSystemFont.DataSource = _SystemFonts
            cbSystemFont.DisplayMember = "Key"
            cbSystemFont.ValueMember = "Value"

            cbFontStyle.DataSource = _FontStyle
            cbFontStyle.DisplayMember = "Key"
            cbFontStyle.ValueMember = "Value"

            cbFontVariant.DataSource = _FontVariant
            cbFontVariant.DisplayMember = "Key"
            cbFontVariant.ValueMember = "Value"

            cbBoldAbsolute.DataSource = _BoldAbsolute
            cbBoldAbsolute.DisplayMember = "Key"
            cbBoldAbsolute.ValueMember = "Value"

            cbBoldRelative.DataSource = _BoldRelative
            cbBoldRelative.DisplayMember = "Key"
            cbBoldRelative.ValueMember = "Value"

            cbCapitalization.DataSource = _TextTransform
            cbCapitalization.DisplayMember = "Key"
            cbCapitalization.ValueMember = "Value"
            '#End Region

            '#Region "set default values"
            rbFamily.Checked = True
            rbSizeSpecific.Checked = True
            rbBoldAbsolute.Checked = True
            '#End Region

            Dim value As String

            '#Region "parse"

            If _dict.TryGetValue("color", value) Then
                txtForeColor.BackColor = ColorTranslator.FromHtml(value)
            End If

            If _dict.TryGetValue("font-style", value) Then
                Dim i As Integer = 0, n As Integer = _FontStyle.Count
                While i < n
                    If String.Equals(value, _FontStyle(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbFontStyle.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("font-size", value) Then
                Dim handled As Boolean = False

                Dim i As Integer = 0, n As Integer = cbAbsoluteSize.Items.Count
                While i < n AndAlso Not handled
                    If String.Equals(value, TryCast(cbAbsoluteSize.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        cbAbsoluteSize.SelectedIndex = i
                        rbSizeAbsolute.Checked = True
                        handled = True
                    End If
                    i += 1
                End While

                i = 0
                n = cbRelativeSize.Items.Count
                While i < n AndAlso Not handled
                    If String.Equals(value, TryCast(cbRelativeSize.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        cbRelativeSize.SelectedIndex = i
                        rbSizeRelative.Checked = True
                        handled = True
                    End If
                    i += 1
                End While

                If Not handled Then
                    rbSizeSpecific.Checked = True
                    i = 0
                    n = cbSpecificSizeType.Items.Count
                    While i < n AndAlso Not handled
                        Dim cbSpecificSizeTypeItem As String = TryCast(cbSpecificSizeType.Items(i), String)
                        If Not String.IsNullOrEmpty(cbSpecificSizeTypeItem) AndAlso value.EndsWith(cbSpecificSizeTypeItem, StringComparison.InvariantCultureIgnoreCase) Then
                            cbSpecificSizeType.SelectedIndex = i
                            tbSpecificSize.Text = value.Substring(0, value.Length - DirectCast(cbSpecificSizeType.Items(i), String).Length)
                            handled = True
                        End If
                        i += 1
                    End While
                End If
            End If

            If _dict.TryGetValue("font-weight", value) Then
                Dim handled As Boolean = False

                Dim i As Integer = 0, n As Integer = _BoldAbsolute.Count
                While i < n AndAlso Not handled
                    If String.Equals(value, _BoldAbsolute(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbBoldAbsolute.SelectedIndex = i
                        rbBoldAbsolute.Checked = True
                        handled = True
                    End If
                    i += 1
                End While

                i = 0
                n = _BoldRelative.Count
                While i < n AndAlso Not handled
                    If String.Equals(value, _BoldRelative(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbBoldRelative.SelectedIndex = i
                        rbBoldRelative.Checked = True
                        handled = True
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("text-decoration", value) Then
                Dim loValue As String = value.ToLowerInvariant()
                cbEffectUnderline.Checked = loValue.Contains("underline")
                cbEffectStrikethrough.Checked = loValue.Contains("line-through") OrElse loValue.Contains("linethrough")
                cbEffectOverline.Checked = loValue.Contains("overline")
                cbEffectNone.Checked = loValue.Contains("none")
            End If

            If _dict.TryGetValue("text-transform", value) Then
                Dim i As Integer = 0, n As Integer = _TextTransform.Count
                While i < n
                    If String.Equals(value, _TextTransform(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbCapitalization.SelectedIndex = i
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("font-family", value) Then
                rbFamily.Checked = True
                tbFontFamily.Text = value
            End If

            If _dict.TryGetValue("font", value) Then
                rbSystemFont.Checked = True

                Dim i As Integer = 0, n As Integer = _SystemFonts.Count
                While i < n
                    If String.Equals((If(value, "")).Replace("-", "").Trim(), _SystemFonts(i).Value.Replace("-", ""), StringComparison.InvariantCultureIgnoreCase) Then
                        cbSystemFont.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If
            '#End Region
        End Sub

#Region "UI handlers"
        ''' <summary>
        ''' Handles the Click event of the btFontFamilySelect control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub btFontFamilySelect_Click(sender As Object, e As EventArgs) Handles btFontFamilySelect.Click
            Using subForm As New frmFontPicker(tbFontFamily.Text)
                If subForm.ShowDialog() = DialogResult.OK Then
                    tbFontFamily.Text = subForm.SelectedFontList
                End If
            End Using
        End Sub

        ''' <summary>
        ''' Fonts the type changed.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub FontTypeChanged(sender As Object, e As EventArgs) Handles rbSystemFont.CheckedChanged, rbFamily.CheckedChanged
            tbFontFamily.Enabled = InlineAssignHelper(btFontFamilySelect.Enabled, rbFamily.Checked)
            cbSystemFont.Enabled = Not rbFamily.Checked

            cbFontStyle.Enabled = InlineAssignHelper(cbFontVariant.Enabled, InlineAssignHelper(gbSize.Enabled, InlineAssignHelper(gbBold.Enabled, rbFamily.Checked)))
        End Sub

        ''' <summary>
        ''' Handles the CheckedChanged event of the rbSizeSpecific control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub rbSizeSpecific_CheckedChanged(sender As Object, e As EventArgs) Handles rbSizeSpecific.CheckedChanged
            tbSpecificSize.Enabled = InlineAssignHelper(cbSpecificSizeType.Enabled, True)
            cbRelativeSize.Enabled = InlineAssignHelper(cbAbsoluteSize.Enabled, False)
        End Sub

        ''' <summary>
        ''' Handles the CheckedChanged event of the rbSizeAbsolute control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub rbSizeAbsolute_CheckedChanged(sender As Object, e As EventArgs) Handles rbSizeAbsolute.CheckedChanged
            cbAbsoluteSize.Enabled = True
            tbSpecificSize.Enabled = InlineAssignHelper(cbSpecificSizeType.Enabled, InlineAssignHelper(cbRelativeSize.Enabled, False))
        End Sub

        ''' <summary>
        ''' Handles the CheckedChanged event of the rbSizeRelative control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub rbSizeRelative_CheckedChanged(sender As Object, e As EventArgs) Handles rbSizeRelative.CheckedChanged
            cbRelativeSize.Enabled = True
            tbSpecificSize.Enabled = InlineAssignHelper(cbSpecificSizeType.Enabled, InlineAssignHelper(cbAbsoluteSize.Enabled, False))
        End Sub

        ''' <summary>
        ''' Handles the CheckedChanged event of the cbEffectNone control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbEffectNone_CheckedChanged(sender As Object, e As EventArgs) Handles cbEffectNone.CheckedChanged
            If cbEffectNone.Checked Then
                cbEffectOverline.Checked = InlineAssignHelper(cbEffectStrikethrough.Checked, InlineAssignHelper(cbEffectUnderline.Checked, False))
                cbEffectOverline.Enabled = InlineAssignHelper(cbEffectStrikethrough.Enabled, InlineAssignHelper(cbEffectUnderline.Enabled, False))
            Else
                cbEffectOverline.Enabled = InlineAssignHelper(cbEffectStrikethrough.Enabled, InlineAssignHelper(cbEffectUnderline.Enabled, True))
            End If
        End Sub

        ''' <summary>
        ''' Rbs the bold radio button changed.
        ''' </summary>
        ''' <param name="sender">The sender.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub rbBoldRadioButtonChanged(sender As Object, e As EventArgs) Handles rbBoldRelative.CheckedChanged, rbBoldAbsolute.CheckedChanged
            cbBoldAbsolute.Enabled = rbBoldAbsolute.Checked
            cbBoldRelative.Enabled = rbBoldRelative.Checked
        End Sub
#End Region

        ''' <summary>
        ''' Handles the Click event of the txtForeColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtForeColor_Click(sender As Object, e As EventArgs) Handles txtForeColor.Click
            If dlgColorPicker.ShowDialog() = DialogResult.OK Then
                txtForeColor.BackColor = dlgColorPicker.Color
            End If
        End Sub

        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function

    End Class
End Namespace