﻿
' ***********************************************************************
' Assembly         : WinFormHtmlEditor
' Author           : SpiceLogic
' Created          : 12-15-2012
'
' Last Modified By : SpiceLogic

' ***********************************************************************
' <copyright file="ucLayout.cs" company="SpiceLogic Inc.">
'     Copyright (c) SpiceLogic Inc.. All rights reserved.
' </copyright>
' <summary></summary>
' ***********************************************************************

Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Windows.Forms

Namespace Dialogs.StyleBuilder
    ''' <summary>
    ''' Class ucLayout
    ''' </summary>
    <ToolboxItem(False)> _
    <FormSelectorPage("Layout", "clip,visibility,display,float,clear,overflow,page-break-before,page-break-after")> _
    Partial Public Class ucLayout
        Inherits UserControl
        Implements IEditorStylePage

        ''' <summary>
        ''' The _dict
        ''' </summary>
        ReadOnly _dict As Dictionary(Of String, String)

#Region "Preset of possible value"

        ''' <summary>
        ''' The _ visibility
        ''' </summary>
        ReadOnly _visibility As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ display
        ''' </summary>
        ReadOnly _display As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ float
        ''' </summary>
        ReadOnly _float As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ clear
        ''' </summary>
        ReadOnly _clear As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ overflow
        ''' </summary>
        ReadOnly _overflow As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ page break before
        ''' </summary>
        ReadOnly _pageBreakBefore As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ page break after
        ''' </summary>
        ReadOnly _pageBreakAfter As List(Of KeyValuePair(Of String, String))
#End Region

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ucLayout"/> class.
        ''' </summary>
        ''' <param name="dict">The dict.</param>
        Public Sub New(dict As Dictionary(Of String, String))
            _dict = dict

            '#Region "Fill lists"
            _visibility.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _visibility.Add(New KeyValuePair(Of String, String)("Hidden", "hidden"))
            _visibility.Add(New KeyValuePair(Of String, String)("Visible", "visible"))

            _display.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _display.Add(New KeyValuePair(Of String, String)("Do not display", "none"))
            _display.Add(New KeyValuePair(Of String, String)("As a block element", "block"))
            _display.Add(New KeyValuePair(Of String, String)("As an inflow element", "inline"))

            _float.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _float.Add(New KeyValuePair(Of String, String)("Don't allow text on sides", "none"))
            _float.Add(New KeyValuePair(Of String, String)("To the right", "right"))
            _float.Add(New KeyValuePair(Of String, String)("To the left", "left"))

            _clear.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _clear.Add(New KeyValuePair(Of String, String)("On either side", "none"))
            _clear.Add(New KeyValuePair(Of String, String)("Only on right", "right"))
            _clear.Add(New KeyValuePair(Of String, String)("Only on left", "left"))
            _clear.Add(New KeyValuePair(Of String, String)("Do not allow", "both"))

            _overflow.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _overflow.Add(New KeyValuePair(Of String, String)("Use scrollbars if needed", "auto"))
            _overflow.Add(New KeyValuePair(Of String, String)("Always use scrollbars", "scroll"))
            _overflow.Add(New KeyValuePair(Of String, String)("Content is not clipped", "visible"))
            _overflow.Add(New KeyValuePair(Of String, String)("Content is clipped", "hidden"))

            _pageBreakBefore.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _pageBreakBefore.Add(New KeyValuePair(Of String, String)("Auto", "auto"))
            _pageBreakBefore.Add(New KeyValuePair(Of String, String)("Force a page break", "always"))
            _pageBreakBefore.Add(New KeyValuePair(Of String, String)("No page break", "avoid"))
            _pageBreakBefore.Add(New KeyValuePair(Of String, String)("Until a blank left page", "left"))
            _pageBreakBefore.Add(New KeyValuePair(Of String, String)("Until a blank right page", "right"))

            _pageBreakAfter = New List(Of KeyValuePair(Of String, String))(_pageBreakBefore)
            '#End Region

            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Flushes the content of the user control back to the dictionary
        ''' </summary>
        Public Sub FlushContent() Implements IEditorStylePage.FlushContent
            _dict.Remove("clip")

            _dict("visibility") = DirectCast(cbVisibility.SelectedValue, String)
            _dict("display") = DirectCast(cbDisplay.SelectedValue, String)
            _dict("float") = DirectCast(cbAllowFloatingObject.SelectedValue, String)
            _dict("clear") = DirectCast(cbAllowTextToFlow.SelectedValue, String)
            _dict("overflow") = DirectCast(cbOverflow.SelectedValue, String)
            _dict("page-break-before") = DirectCast(cbPbBefore.SelectedValue, String)
            _dict("page-break-after") = DirectCast(cbPbAfter.SelectedValue, String)

            If True Then
                ' clip
                Dim top As String = tbTop.Text.Trim()
                Dim right As String = tbRight.Text.Trim()
                Dim bottom As String = tbBottom.Text.Trim()
                Dim left As String = tbLeft.Text.Trim()

                If top.Length + right.Length + bottom.Length + left.Length > 0 Then
                    ' fix values if any
                    top = If(top.Length = 0, "auto", top + cbTopType.Text)

                    right = If(right.Length = 0, "auto", right + cbRightType.Text)

                    bottom = If(bottom.Length = 0, "auto", bottom + cbBottomType.Text)

                    left = If(left.Length = 0, "auto", left + cbLeftType.Text)

                    ' store to the dictionary
                    _dict("clip") = String.Format("rect({0} {1} {2} {3})", top, right, bottom, left)
                End If
            End If
        End Sub

        ''' <summary>
        ''' Handles the Load event of the ucLayout control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub ucLayout_Load(sender As Object, e As EventArgs) Handles Me.Load
            '#Region "set data sources"
            cbVisibility.DataSource = _visibility
            cbVisibility.DisplayMember = "Key"
            cbVisibility.ValueMember = "Value"

            cbDisplay.DataSource = _display
            cbDisplay.DisplayMember = "Key"
            cbDisplay.ValueMember = "Value"

            cbAllowTextToFlow.DataSource = _float
            cbAllowTextToFlow.DisplayMember = "Key"
            cbAllowTextToFlow.ValueMember = "Value"

            cbAllowFloatingObject.DataSource = _clear
            cbAllowFloatingObject.DisplayMember = "Key"
            cbAllowFloatingObject.ValueMember = "Value"

            cbOverflow.DataSource = _overflow
            cbOverflow.DisplayMember = "Key"
            cbOverflow.ValueMember = "Value"

            cbPbBefore.DataSource = _pageBreakBefore
            cbPbBefore.DisplayMember = "Key"
            cbPbBefore.ValueMember = "Value"

            cbPbAfter.DataSource = _pageBreakAfter
            cbPbAfter.DisplayMember = "Key"
            cbPbAfter.ValueMember = "Value"
            '#End Region

            Dim value As String

            '#Region "parse dictionary's values"
            If _dict.TryGetValue("overflow", value) Then
                Dim i As Integer = 0, n As Integer = _overflow.Count
                While i < n
                    If String.Equals(value, _overflow(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbOverflow.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("visibility", value) Then
                Dim i As Integer = 0, n As Integer = _visibility.Count
                While i < n
                    If String.Equals(value, _visibility(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbVisibility.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("display", value) Then
                Dim i As Integer = 0, n As Integer = _display.Count
                While i < n
                    If String.Equals(value, _display(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbDisplay.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("float", value) Then
                Dim i As Integer = 0, n As Integer = _float.Count
                While i < n
                    If String.Equals(value, _float(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbAllowTextToFlow.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("clear", value) Then
                Dim i As Integer = 0, n As Integer = _clear.Count
                While i < n
                    If String.Equals(value, _clear(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbAllowFloatingObject.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("page-break-before", value) Then
                Dim i As Integer = 0, n As Integer = _pageBreakBefore.Count
                While i < n
                    If String.Equals(value, _pageBreakBefore(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbPbBefore.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("page-break-after", value) Then
                Dim i As Integer = 0, n As Integer = _pageBreakAfter.Count
                While i < n
                    If String.Equals(value, _pageBreakAfter(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbPbAfter.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("clip", value) Then
                If value.StartsWith("rect(", StringComparison.InvariantCultureIgnoreCase) AndAlso value.EndsWith(")") Then
                    Dim inner As String = value.Substring(5, value.Length - 6)
                    Dim parts As String() = inner.Split(" "c)
                    If parts.Length >= 4 Then
                        Dim top As String = parts(0)
                        Dim right As String = parts(1)
                        Dim bottom As String = parts(2)
                        Dim left As String = parts(3)

                        Dim i As Integer = 0
                        Dim n As Integer = cbTopType.Items.Count
                        While i < n
                            If top.EndsWith(DirectCast(cbTopType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                tbTop.Text = top.Substring(0, top.Length - DirectCast(cbTopType.Items(i), String).Length)
                                cbTopType.SelectedIndex = i
                                Exit While
                            End If
                            i += 1
                        End While

                        i = 0
                        n = cbRightType.Items.Count
                        While i < n
                            If right.EndsWith(DirectCast(cbRightType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                tbRight.Text = right.Substring(0, right.Length - DirectCast(cbRightType.Items(i), String).Length)
                                cbRightType.SelectedIndex = i
                                Exit While
                            End If
                            i += 1
                        End While

                        i = 0
                        n = cbBottomType.Items.Count
                        While i < n
                            If bottom.EndsWith(DirectCast(cbBottomType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                tbBottom.Text = bottom.Substring(0, bottom.Length - DirectCast(cbBottomType.Items(i), String).Length)
                                cbBottomType.SelectedIndex = i
                                Exit While
                            End If
                            i += 1
                        End While

                        i = 0
                        n = cbLeftType.Items.Count
                        While i < n
                            If left.EndsWith(DirectCast(cbLeftType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                                tbLeft.Text = left.Substring(0, left.Length - DirectCast(cbLeftType.Items(i), String).Length)
                                cbLeftType.SelectedIndex = i
                                Exit While
                            End If
                            i += 1
                        End While
                    End If
                End If
            End If
            '#End Region
        End Sub

    End Class
End Namespace