﻿
' ***********************************************************************
' Assembly         : WinFormHtmlEditor
' Author           : SpiceLogic
' Created          : 12-15-2012
'
' Last Modified By : SpiceLogic

' ***********************************************************************
' <copyright file="ucPosition.cs" company="SpiceLogic Inc.">
'     Copyright (c) SpiceLogic Inc.. All rights reserved.
' </copyright>
' <summary></summary>
' ***********************************************************************

Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Windows.Forms

Namespace Dialogs.StyleBuilder
    ''' <summary>
    ''' Class ucPosition
    ''' </summary>
    <ToolboxItem(False)> _
    <FormSelectorPage("Position", "position,top,left,width,height,z-index")> _
    Partial Public Class ucPosition
        Inherits UserControl
        Implements IEditorStylePage

        ''' <summary>
        ''' The _dict
        ''' </summary>
        ReadOnly _dict As Dictionary(Of String, String)

#Region "Preset of possible values"

        ''' <summary>
        ''' The _ position
        ''' </summary>
        ReadOnly _position As New List(Of KeyValuePair(Of String, String))()
#End Region

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ucPosition"/> class.
        ''' </summary>
        ''' <param name="dict">The dict.</param>
        Public Sub New(dict As Dictionary(Of String, String))
            _dict = dict

            '#Region "Initialize presets"
            _position.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _position.Add(New KeyValuePair(Of String, String)("Position in normal flow", "static"))
            _position.Add(New KeyValuePair(Of String, String)("Offset from normal flow", "relative"))
            _position.Add(New KeyValuePair(Of String, String)("Absolutely position", "absolute"))
            '#End Region

            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Flushes the content of the user control back to the dictionary
        ''' </summary>
        Public Sub FlushContent() Implements IEditorStylePage.FlushContent
            _dict.Remove("position")
            _dict.Remove("top")
            _dict.Remove("left")
            _dict.Remove("width")
            _dict.Remove("height")
            _dict.Remove("z-index")

            If cbPositionMode.SelectedIndex >= 2 Then
                If tbLeft.Text.Trim().Length > 0 Then
                    _dict("left") = tbLeft.Text + cbLeftType.Text
                End If
                If tbTop.Text.Trim().Length > 0 Then
                    _dict("top") = tbTop.Text + cbTopType.Text
                End If

                If cbPositionMode.SelectedIndex = 3 Then
                    _dict("z-index") = tbZIndex.Text
                End If
            End If

            If tbHeight.Text.Trim().Length > 0 Then
                _dict("height") = tbHeight.Text + cbHeightType.Text
            End If
            If tbWidth.Text.Trim().Length > 0 Then
                _dict("width") = tbWidth.Text + cbWidthType.Text
            End If

            _dict("position") = DirectCast(cbPositionMode.SelectedValue, String)
        End Sub

        ''' <summary>
        ''' Handles the Load event of the ucPosition control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub ucPosition_Load(sender As Object, e As EventArgs) Handles Me.Load
            '#Region "set data sources"
            cbPositionMode.DataSource = _position
            cbPositionMode.DisplayMember = "Key"
            cbPositionMode.ValueMember = "Value"
            '#End Region

            Dim value As String

            '#Region "parse"
            If _dict.TryGetValue("left", value) Then
                Dim i As Integer = 0, n As Integer = cbLeftType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbLeftType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        tbLeft.Text = value.Substring(0, value.Length - DirectCast(cbLeftType.Items(i), String).Length)
                        cbLeftType.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("top", value) Then
                Dim i As Integer = 0, n As Integer = cbTopType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbTopType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        tbTop.Text = value.Substring(0, value.Length - DirectCast(cbTopType.Items(i), String).Length)
                        cbTopType.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("width", value) Then
                Dim i As Integer = 0, n As Integer = cbWidthType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbWidthType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        tbWidth.Text = value.Substring(0, value.Length - DirectCast(cbWidthType.Items(i), String).Length)
                        cbWidthType.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("height", value) Then
                Dim i As Integer = 0, n As Integer = cbHeightType.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbHeightType.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        tbHeight.Text = value.Substring(0, value.Length - DirectCast(cbHeightType.Items(i), String).Length)
                        cbHeightType.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("z-index", value) Then
                tbZIndex.Text = value
            End If

            If _dict.TryGetValue("position", value) Then
                Dim i As Integer = 0, n As Integer = _position.Count
                While i < n
                    If String.Equals(value, _position(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbPositionMode.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If
            '#End Region
        End Sub

#Region "UI handlers"
        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbPositionMode control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbPositionMode_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbPositionMode.SelectedIndexChanged
            tbLeft.Enabled = InlineAssignHelper(cbLeftType.Enabled, InlineAssignHelper(tbTop.Enabled, InlineAssignHelper(cbTopType.Enabled, cbPositionMode.SelectedIndex >= 2)))

            tbZIndex.Enabled = cbPositionMode.SelectedIndex = 3
        End Sub

        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function
#End Region

    End Class
End Namespace