﻿
' ***********************************************************************
' Assembly         : WinFormHtmlEditor
' Author           : SpiceLogic
' Created          : 12-15-2012
'
' Last Modified By : SpiceLogic

' ***********************************************************************
' <copyright file="ucText.cs" company="SpiceLogic Inc.">
'     Copyright (c) SpiceLogic Inc.. All rights reserved.
' </copyright>
' <summary></summary>
' ***********************************************************************

Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Windows.Forms

Namespace Dialogs.StyleBuilder
    ''' <summary>
    ''' Class ucText
    ''' </summary>
    <ToolboxItem(False)> _
    <FormSelectorPage("Text", "text-align,vertical-align,text-justify,letter-spacing,line-height,direction,text-indent")> _
    Partial Public Class ucText
        Inherits UserControl
        Implements IEditorStylePage

        ''' <summary>
        ''' The _dict
        ''' </summary>
        ReadOnly _dict As Dictionary(Of String, String)

#Region "Preset of possible values"

        ''' <summary>
        ''' The _ text align
        ''' </summary>
        ReadOnly _textAlign As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ vertical align
        ''' </summary>
        ReadOnly _verticalAlign As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ text justify
        ''' </summary>
        ReadOnly _textJustify As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ letter spacing
        ''' </summary>
        ReadOnly _letterSpacing As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ line height
        ''' </summary>
        ReadOnly _lineHeight As New List(Of KeyValuePair(Of String, String))()
        ''' <summary>
        ''' The _ direction
        ''' </summary>
        ReadOnly _direction As New List(Of KeyValuePair(Of String, String))()

#End Region

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ucText"/> class.
        ''' </summary>
        ''' <param name="dict">The dict.</param>
        Public Sub New(dict As Dictionary(Of String, String))
            _dict = dict

            '#Region "Initialize presets"
            _textAlign.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _textAlign.Add(New KeyValuePair(Of String, String)("Left", "left"))
            _textAlign.Add(New KeyValuePair(Of String, String)("Center", "center"))
            _textAlign.Add(New KeyValuePair(Of String, String)("Right", "right"))
            _textAlign.Add(New KeyValuePair(Of String, String)("Justified", "justify"))

            _verticalAlign.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("baseline", "baseline"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("sub", "sub"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("super", "super"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("top", "top"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("text-top", "text-top"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("middle", "middle"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("bottom", "bottom"))
            _verticalAlign.Add(New KeyValuePair(Of String, String)("text-bottom", "text-bottom"))

            _textJustify.Add(New KeyValuePair(Of String, String)("", ""))
            _textJustify.Add(New KeyValuePair(Of String, String)("Auto", "auto"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Space words", "inter-word"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Newspaper style", "newspaper"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Distribute spacing", "distribute"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Distribute all lines", "dibtribute-all-lines"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Inter-cluster", "inter-cluster"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Inter-ideograph", "inter-ideograph"))
            _textJustify.Add(New KeyValuePair(Of String, String)("Kashida", "kashida"))

            _letterSpacing.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _letterSpacing.Add(New KeyValuePair(Of String, String)("Normal", "normal"))
            _letterSpacing.Add(New KeyValuePair(Of String, String)("Custom", ""))

            _lineHeight.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _lineHeight.Add(New KeyValuePair(Of String, String)("Normal", "normal"))
            _lineHeight.Add(New KeyValuePair(Of String, String)("Custom", ""))

            _direction.Add(New KeyValuePair(Of String, String)("<Not Set>", ""))
            _direction.Add(New KeyValuePair(Of String, String)("Left to right", "ltr"))
            _direction.Add(New KeyValuePair(Of String, String)("Right to left", "rtl"))
            '#End Region

            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Flushes the content of the user control back to the dictionary
        ''' </summary>
        Public Sub FlushContent() Implements IEditorStylePage.FlushContent
            ' remove previous entries
            _dict.Remove("text-align")
            _dict.Remove("vertical-align")
            _dict.Remove("text-justify")

            _dict.Remove("letter-spacing")
            _dict.Remove("line-height")

            _dict.Remove("direction")
            _dict.Remove("text-indent")

            ' save form's data
            _dict("text-align") = DirectCast(cbAlHorizontal.SelectedValue, String)
            If cbAlHorizontal.SelectedIndex = 4 Then
                _dict("text-justify") = DirectCast(cbAlJustification.SelectedValue, String)
            End If
            _dict("vertical-align") = DirectCast(cbAlVertical.SelectedValue, String)

            _dict("letter-spacing") = If(cbSpacingLetters.SelectedIndex <> 2, DirectCast(cbSpacingLetters.SelectedValue, String), tbSpacingLetters.Text + cbSpacingLettersCustom.Text)

            _dict("line-height") = If(cbSpacingLines.SelectedIndex <> 2, DirectCast(cbSpacingLines.SelectedValue, String), tbSpacingLines.Text + cbSpacingLinesCustom.Text)

            If tbTextFlowIndentation.Text.Trim().Length > 0 Then
                _dict("text-indent") = tbTextFlowIndentation.Text + cbTextFlowCustom.Text
            End If

            _dict("direction") = DirectCast(cbTextFlowDirection.SelectedValue, String)
        End Sub

        ''' <summary>
        ''' Handles the Load event of the ucText control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub ucText_Load(sender As Object, e As EventArgs) Handles Me.Load
            '#Region "set data sources"
            cbAlHorizontal.DataSource = _textAlign
            cbAlHorizontal.DisplayMember = "Key"
            cbAlHorizontal.ValueMember = "Value"

            cbAlVertical.DataSource = _verticalAlign
            cbAlVertical.DisplayMember = "Key"
            cbAlVertical.ValueMember = "Value"

            cbAlJustification.DataSource = _textJustify
            cbAlJustification.DisplayMember = "Key"
            cbAlJustification.ValueMember = "Value"

            cbSpacingLetters.DataSource = _letterSpacing
            cbSpacingLetters.DisplayMember = "Key"
            cbSpacingLetters.ValueMember = "Value"

            cbSpacingLines.DataSource = _lineHeight
            cbSpacingLines.DisplayMember = "Key"
            cbSpacingLines.ValueMember = "Value"

            cbTextFlowDirection.DataSource = _direction
            cbTextFlowDirection.DisplayMember = "Key"
            cbTextFlowDirection.ValueMember = "Value"
            '#End Region

            Dim value As String

            '#Region "parse alignment"
            If _dict.TryGetValue("vertical-align", value) Then
                Dim i As Integer = 0, n As Integer = _verticalAlign.Count
                While i < n
                    If String.Equals(value, _verticalAlign(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbAlVertical.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("text-justify", value) Then
                Dim i As Integer = 0, n As Integer = _textJustify.Count
                While i < n
                    If String.Equals(value, _textJustify(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbAlJustification.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("text-align", value) Then
                Dim i As Integer = 0, n As Integer = _textAlign.Count
                While i < n
                    If String.Equals(value, _textAlign(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbAlHorizontal.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If
            '#End Region

            '#Region "parse spacing"
            If _dict.TryGetValue("letter-spacing", value) Then
                Dim handled As Boolean = False

                Dim i As Integer = 0, n As Integer = _letterSpacing.Count
                While i < n AndAlso Not handled
                    If value.Equals(_letterSpacing(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbSpacingLetters.SelectedIndex = i
                        handled = True
                    End If
                    i += 1
                End While

                If Not handled Then
                    i = 0
                    n = cbSpacingLettersCustom.Items.Count
                    While i < n
                        If value.EndsWith(DirectCast(cbSpacingLettersCustom.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                            tbSpacingLetters.Text = value.Substring(0, value.Length - DirectCast(cbSpacingLettersCustom.Items(i), String).Length)
                            cbSpacingLettersCustom.SelectedIndex = i
                            cbSpacingLetters.SelectedIndex = 2
                            Exit While
                        End If
                        i += 1
                    End While
                End If
            End If

            If _dict.TryGetValue("line-height", value) Then
                Dim handled As Boolean = False

                Dim i As Integer = 0, n As Integer = _lineHeight.Count
                While i < n AndAlso Not handled
                    If value.Equals(_lineHeight(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbSpacingLines.SelectedIndex = i
                        handled = True
                    End If
                    i += 1
                End While

                If Not handled Then
                    i = 0
                    n = cbSpacingLinesCustom.Items.Count
                    While i < n
                        If value.EndsWith(DirectCast(cbSpacingLinesCustom.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                            tbSpacingLines.Text = value.Substring(0, value.Length - DirectCast(cbSpacingLinesCustom.Items(i), String).Length)
                            cbSpacingLinesCustom.SelectedIndex = i
                            cbSpacingLines.SelectedIndex = 2
                            Exit While
                        End If
                        i += 1
                    End While
                End If
            End If
            '#End Region

            '#Region "parse text flow"
            If _dict.TryGetValue("text-indent", value) Then
                Dim i As Integer = 0, n As Integer = cbTextFlowCustom.Items.Count
                While i < n
                    If value.EndsWith(DirectCast(cbTextFlowCustom.Items(i), String), StringComparison.InvariantCultureIgnoreCase) Then
                        tbTextFlowIndentation.Text = value.Substring(0, value.Length - DirectCast(cbTextFlowCustom.Items(i), String).Length)
                        cbTextFlowCustom.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If

            If _dict.TryGetValue("direction", value) Then
                Dim i As Integer = 0, n As Integer = _direction.Count
                While i < n
                    If value.Equals(_direction(i).Value, StringComparison.InvariantCultureIgnoreCase) Then
                        cbTextFlowDirection.SelectedIndex = i
                        Exit While
                    End If
                    i += 1
                End While
            End If
            '#End Region
        End Sub

#Region "UI handling"
        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbAlHorizontal control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbAlHorizontal_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbAlHorizontal.SelectedIndexChanged
            cbAlJustification.Enabled = cbAlHorizontal.SelectedIndex = 4
        End Sub

        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbSpacingLetters control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbSpacingLetters_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbSpacingLetters.SelectedIndexChanged
            tbSpacingLetters.Enabled = InlineAssignHelper(cbSpacingLettersCustom.Enabled, cbSpacingLetters.SelectedIndex = 2)
        End Sub

        ''' <summary>
        ''' Handles the SelectedIndexChanged event of the cbSpacingLines control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub cbSpacingLines_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cbSpacingLines.SelectedIndexChanged
            tbSpacingLines.Enabled = InlineAssignHelper(cbSpacingLinesCustom.Enabled, cbSpacingLines.SelectedIndex = 2)
        End Sub

        Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function
#End Region

    End Class
End Namespace