﻿
Imports System.Collections.Generic
Imports System.Collections.Specialized
Imports System.Drawing
Imports System.Globalization
Imports System.Windows.Forms
Imports SpiceLogic.HtmlEditorControl.Domain.BOs.Entities
Imports SpiceLogic.HtmlEditorControl.Domain.Dialogs

Namespace Dialogs
    ''' <summary>
    ''' Class TableCellPropertiesDialog
    ''' </summary>
    Partial Public Class TableCellPropertiesDialog
        Inherits Form
        Implements ITableCellDialog

        ''' <summary>
        ''' The _element
        ''' </summary>
        Private _element As TableCellElement

        Private ReadOnly _propertiesAffected As New List(Of String)() From { _
            "BgColor", _
            "CSSClassName", _
            "Width", _
            "WidthUnit", _
            "Height", _
            "HeightUnit", _
            "HorizontalAlign", _
            "VerticalAlign", _
            "NoWrap" _
        }

        ''' <summary>
        ''' Initializes a new instance of the <see cref="TableCellPropertiesDialog" /> class.
        ''' </summary>
        Public Sub New()
            InitializeComponent()
            Me.CancelButton = btnCancel
        End Sub

        ''' <summary>
        ''' Gets or sets the element.
        ''' </summary>
        ''' <value>The element.</value>
        Public Property Element() As TableCellElement Implements ITableCellDialog.Element
            Get
                _element.ResetStyles(Me._propertiesAffected)

                '                _element.BgColor = chkBgColor.Checked ? ColorTranslator.ToHtml(txtBgColor.BackColor) : null;
                _element.BgColor = If(chklnkBgColor.Checked, ColorTranslator.ToHtml(txtBgColor.BackColor), Nothing)
                _element.CSSClassName = txtClassName.Text.Trim()
                _element.CSSStyle = txtCss.Text.Trim()

                If chkWidth.Checked AndAlso cmbWidthUnit.SelectedItem IsNot Nothing Then
                    _element.Width = Convert.ToInt32(txtWidth.Text.Trim())
                    _element.WidthUnit = cmbWidthUnit.SelectedItem.ToString()
                End If

                If chkHeight.Checked AndAlso cmbHeightUnit.SelectedItem IsNot Nothing Then
                    _element.Height = Convert.ToInt32(txtHeight.Text.Trim())
                    _element.HeightUnit = cmbHeightUnit.SelectedItem.ToString()
                End If

                If cmbHorizontalAlign.SelectedItem IsNot Nothing Then
                    _element.HorizontalAlign = cmbHorizontalAlign.SelectedItem.ToString()
                End If

                If cmbVerticalAlign.SelectedItem IsNot Nothing Then
                    _element.VerticalAlign = cmbVerticalAlign.SelectedItem.ToString()
                End If
                _element.NoWrap = chkNoWrap.Checked
                _element.OverrideSettingsToAllCells = chkOverrideSettings4Cells.Checked
                Return _element
            End Get
            Set(value As TableCellElement)
                _element = value

                If _element Is Nothing Then
                    Return
                End If

                chklnkBgColor.Checked = Not String.IsNullOrEmpty(_element.BgColor)
                If chklnkBgColor.Checked Then
                    Dim converter As New ColorConverter()
                    Dim convertFromString As Object = converter.ConvertFromString(_element.BgColor)
                    If convertFromString IsNot Nothing Then
                        txtBgColor.BackColor = DirectCast(convertFromString, Color)
                    End If
                End If

                txtClassName.Text = _element.CSSClassName
                chkWidth.Checked = _element.Width.HasValue
                If chkWidth.Checked AndAlso _element.WidthUnit IsNot Nothing Then
                    If _element.Width.HasValue Then
                        txtWidth.Text = _element.Width.Value.ToString(CultureInfo.InvariantCulture)
                    End If
                    cmbWidthUnit.SelectedItem = _element.WidthUnit
                End If

                chkHeight.Checked = _element.Height.HasValue
                If chkHeight.Checked AndAlso _element.HeightUnit IsNot Nothing Then
                    If _element.Height.HasValue Then
                        txtHeight.Text = _element.Height.Value.ToString(CultureInfo.InvariantCulture)
                    End If
                    cmbHeightUnit.SelectedItem = _element.HeightUnit
                End If

                If _element.HorizontalAlign IsNot Nothing Then
                    cmbHorizontalAlign.SelectedItem = _element.HorizontalAlign
                End If

                If _element.VerticalAlign IsNot Nothing Then
                    cmbVerticalAlign.SelectedItem = _element.VerticalAlign
                End If
                chkNoWrap.Checked = _element.NoWrap
                chkOverrideSettings4Cells.Checked = _element.OverrideSettingsToAllCells

                txtCss.Text = _element.GetCssStyleWithoutProperties(Me._propertiesAffected)
            End Set
        End Property
        ''' <summary>
        ''' Gets the cell attribute collection.
        ''' </summary>
        ''' <value>The cell attribute collection.</value>
        Public ReadOnly Property CellAttributeCollection() As NameValueCollection
            Get
                Dim myColl As New NameValueCollection()

                If chkWidth.Checked Then
                    myColl.Add("width", String.Concat(txtWidth.Text.Trim(), cmbWidthUnit.Text.Trim()))
                End If

                If chkHeight.Checked Then
                    myColl.Add("height", String.Concat(txtHeight.Text.Trim(), cmbHeightUnit.Text.Trim()))
                End If

                If chklnkBgColor.Checked Then
                    myColl.Add("bgcolor", ColorTranslator.ToHtml(txtBgColor.BackColor))
                End If

                If cmbHorizontalAlign.SelectedIndex <> 0 Then
                    myColl.Add("align", cmbHorizontalAlign.Text)
                End If

                If cmbVerticalAlign.SelectedIndex <> 0 Then
                    myColl.Add("valign", cmbVerticalAlign.Text)
                End If

                If chkNoWrap.Checked Then
                    myColl.Add("nowrap", "nowrap")
                End If

                Return myColl
            End Get
        End Property

        ''' <summary>
        ''' Gets the table cell attribute string.
        ''' </summary>
        ''' <value>The table cell attribute string.</value>
        Public ReadOnly Property TableCellAttributeString() As String
            Get
                Dim tableCellAttributes As String() = New String(-1) {}
                Dim tableCellStyleAttributes As String() = New String(-1) {}

                If chkWidth.Checked Then
                    Array.Resize(tableCellStyleAttributes, tableCellStyleAttributes.Length + 1)
                    tableCellStyleAttributes(tableCellStyleAttributes.Length - 1) = String.Format("width: {0}{1}", txtWidth.Text.Trim(), cmbWidthUnit.Text.Trim())
                End If

                If chkHeight.Checked Then
                    Array.Resize(tableCellStyleAttributes, tableCellStyleAttributes.Length + 1)
                    tableCellStyleAttributes(tableCellStyleAttributes.Length - 1) = String.Format("height: {0}{1}", txtHeight.Text.Trim(), cmbHeightUnit.Text.Trim())
                End If

                If chklnkBgColor.Checked Then
                    Array.Resize(tableCellStyleAttributes, tableCellStyleAttributes.Length + 1)
                    tableCellStyleAttributes(tableCellStyleAttributes.Length - 1) = String.Format("background-color: {0}", ColorTranslator.ToHtml(txtBgColor.BackColor))
                End If

                If tableCellStyleAttributes.Length <> 0 Then
                    Dim tableCellStyleText As String = String.Format("style = ""{0}""", String.Join("; ", tableCellStyleAttributes))
                    Array.Resize(tableCellAttributes, tableCellAttributes.Length + 1)
                    tableCellAttributes(tableCellAttributes.Length - 1) = tableCellStyleText
                End If

                If cmbHorizontalAlign.SelectedIndex <> 0 Then
                    Array.Resize(tableCellAttributes, tableCellAttributes.Length + 1)
                    tableCellAttributes(tableCellAttributes.Length - 1) = String.Format("align=""{0}""", cmbHorizontalAlign.Text)
                End If

                If cmbVerticalAlign.SelectedIndex <> 0 Then
                    Array.Resize(tableCellAttributes, tableCellAttributes.Length + 1)
                    tableCellAttributes(tableCellAttributes.Length - 1) = String.Format("valign=""{0}""", cmbVerticalAlign.Text)
                End If

                If chkNoWrap.Checked Then
                    Array.Resize(tableCellAttributes, tableCellAttributes.Length + 1)
                    tableCellAttributes(tableCellAttributes.Length - 1) = "nowrap=""nowrap"""
                End If

                If Not String.IsNullOrEmpty(txtClassName.Text.Trim()) Then
                    Array.Resize(tableCellAttributes, tableCellAttributes.Length + 1)
                    tableCellAttributes(tableCellAttributes.Length - 1) = "class=""" + txtClassName.Text.Trim() + """"
                End If

                Return String.Join(" ", tableCellAttributes)
            End Get
        End Property

        ''' <summary>
        ''' Get a lock for a property [override cell attributes].
        ''' </summary>
        Public Function LockOverrideSettingsToAllCells() As IDisposable Implements ITableCellDialog.LockOverrideSettingsToAllCells
            Return New lockOverrideSettingsToAllCellsClass(Me)
        End Function

        ''' <summary>
        ''' Gets a value indicating whether [override cell attributes].
        ''' </summary>
        ''' <value><c>true</c> if [override cell attributes]; otherwise, <c>false</c>.</value>
        Public ReadOnly Property OverrideCellAttributes() As Boolean
            Get
                Return chkOverrideSettings4Cells.Checked
            End Get
        End Property

#Region "--------------- UI Event Handlers ------------------"

        ''' <summary>
        ''' Handles the CheckedChanged event of the chkWidth control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        Private Sub chkWidth_CheckedChanged(sender As Object, e As EventArgs) Handles chkWidth.CheckedChanged
            txtWidth.Enabled = chkWidth.Checked
            cmbWidthUnit.Enabled = chkWidth.Checked
        End Sub

        ''' <summary>
        ''' Handles the CheckedChanged event of the chkHeight control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        Private Sub chkHeight_CheckedChanged(sender As Object, e As EventArgs) Handles chkHeight.CheckedChanged
            txtHeight.Enabled = chkHeight.Checked
            cmbHeightUnit.Enabled = chkHeight.Checked
        End Sub

#End Region

        Private Class lockOverrideSettingsToAllCellsClass
            Implements IDisposable

            Private ReadOnly _dialog As TableCellPropertiesDialog
            Private ReadOnly _initallyEnabled As Boolean

            Public Sub New(dialog As TableCellPropertiesDialog)
                _dialog = dialog
                _initallyEnabled = _dialog.chkOverrideSettings4Cells.Enabled
                _dialog.chkOverrideSettings4Cells.Enabled = False
            End Sub

            Public Sub Dispose() Implements IDisposable.Dispose
                _dialog.chkOverrideSettings4Cells.Enabled = _initallyEnabled
            End Sub

        End Class

        ''' <summary>
        ''' Handles the LinkClicked event of the chklnkBgColor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.Windows.Forms.LinkLabelLinkClickedEventArgs" /> instance containing the event data.</param>
        Private Sub chklnkBgColor_LinkClicked(sender As Object, e As LinkLabelLinkClickedEventArgs) Handles chklnkBgColor.LinkClicked
            showDialogForChoosingColor()
        End Sub

        Private Sub showDialogForChoosingColor()
            Using myDialog As New ColorDialog() With { _
                .AllowFullOpen = True, _
                .AnyColor = True, _
                .Color = Me.txtBgColor.BackColor, _
                .CustomColors = {ColorTranslator.ToOle(Me.txtBgColor.BackColor)} _
            }
                If myDialog.ShowDialog() = DialogResult.OK Then
                    Me.txtBgColor.BackColor = myDialog.Color
                End If
            End Using
        End Sub

        ''' <summary>
        ''' Shows current form as a dialog
        ''' </summary>
        ''' <returns>Returns <see cref="System.Windows.Forms.DialogResult"/></returns>
        Public Overloads Function ShowDialog() As DialogResult Implements IDialog.ShowDialog
            Return MyBase.ShowDialog()
        End Function

    End Class
End Namespace