﻿
' ***********************************************************************
' Assembly         : WinFormHtmlEditor
' Author           : SpiceLogic
' Created          : 12-15-2012
'
' Last Modified By : SpiceLogic

' ***********************************************************************
' <copyright file="YouTubeVideoInsertDialog.cs" company="SpiceLogic Inc.">
'     Copyright (c) SpiceLogic Inc.. All rights reserved.
' </copyright>
' <summary></summary>
' ***********************************************************************

Imports System.Windows.Forms
Imports SpiceLogic.HtmlEditorControl.Domain.BOs.Entities
Imports SpiceLogic.HtmlEditorControl.Domain.Dialogs

Namespace Dialogs
    ''' <summary>
    ''' Class YouTubeVideoInsertDialog
    ''' </summary>
    Partial Public Class YouTubeVideoInsertDialog
        Inherits Form
        Implements IYouTubeVideoInsertDialog

        ''' <summary>
        ''' The _the original element
        ''' </summary>
        Private _theOriginalElement As YouTubeVideoElement

        ''' <summary>
        ''' Initializes a new instance of the <see cref="YouTubeVideoInsertDialog" /> class.
        ''' </summary>
        Public Sub New()
            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Gets or sets the element.
        ''' </summary>
        ''' <value>The element.</value>
        Public Property Element() As YouTubeVideoElement Implements IYouTubeVideoInsertDialog.Element
            Get
                If _theOriginalElement Is Nothing Then
                    Dim theElement As New YouTubeVideoElement() With { _
                        .URL = txtUrl.Text.Trim(), _
                        .Width = txtWidth.Text, _
                        .Height = txtHeight.Text _
                    }
                    Return theElement
                End If
                _theOriginalElement.URL = txtUrl.Text.Trim()
                _theOriginalElement.Width = txtWidth.Text
                _theOriginalElement.Height = txtHeight.Text
                _theOriginalElement.CSSStyle = txtCssStyle.Text
                Return _theOriginalElement
            End Get
            Set(value As YouTubeVideoElement)
                Me._theOriginalElement = value
                txtUrl.Text = value.URL
                txtHeight.Text = value.Height
                txtWidth.Text = value.Width
                txtCssStyle.Text = value.CSSStyle
            End Set
        End Property

        ''' <summary>
        ''' Handles the Click event of the btnOK control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        Private Sub btnOK_Click(sender As Object, e As EventArgs) Handles btnOK.Click
            If String.IsNullOrEmpty(txtUrl.Text) Then
                MessageBox.Show("The YouTube URL cannot be empty.")
                txtUrl.Focus()
                Return
            End If

            Dim theUrl As String = txtUrl.Text.Trim()
            If theUrl = String.Empty Then
                MessageBox.Show("The YouTube URL cannot be empty.")
                txtUrl.Focus()
                Return
            End If

            If Not theUrl.ToLower().Contains("youtube.com") Then
                MessageBox.Show("The URL you provided does not contain the YouTube Domain name", "Invalid URL")
                Return
            End If

            Me.DialogResult = DialogResult.OK
        End Sub

        ''' <summary>
        ''' Shows current form as a dialog
        ''' </summary>
        ''' <returns>Returns <see cref="System.Windows.Forms.DialogResult"/></returns>
        Public Overloads Function ShowDialog() As DialogResult Implements IDialog.ShowDialog
            Return MyBase.ShowDialog()
        End Function

    End Class
End Namespace